# MIT License
#
# Copyright (c) 2021-2025 Advanced Micro Devices, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


# This file has been autogenerated, do not modify.

from libc.stdint cimport *
ctypedef bint _Bool # bool is not a reserved keyword in C, _Bool is
# from .chip cimport * # via chipblas
from .chipblas cimport *
cdef extern from "hipsolver/hipsolver.h":

    cdef int hipsolverVersionMajor

    cdef int hipsolverVersionMinor

    cdef int hipsolverVersionPatch

    ctypedef short __int16_t

    ctypedef unsigned short __uint16_t

    ctypedef enum:
        FP_NAN
        FP_INFINITE
        FP_ZERO
        FP_SUBNORMAL
        FP_NORMAL

    ctypedef enum:
        HIP_SUCCESS
        HIP_ERROR_INVALID_VALUE
        HIP_ERROR_NOT_INITIALIZED
        HIP_ERROR_LAUNCH_OUT_OF_RESOURCES

    ctypedef void * hipsolverHandle_t

    ctypedef void * hipsolverGesvdjInfo_t

    ctypedef void * hipsolverSyevjInfo_t

    ctypedef enum hipsolverStatus_t:
        HIPSOLVER_STATUS_SUCCESS
        HIPSOLVER_STATUS_NOT_INITIALIZED
        HIPSOLVER_STATUS_ALLOC_FAILED
        HIPSOLVER_STATUS_INVALID_VALUE
        HIPSOLVER_STATUS_MAPPING_ERROR
        HIPSOLVER_STATUS_EXECUTION_FAILED
        HIPSOLVER_STATUS_INTERNAL_ERROR
        HIPSOLVER_STATUS_NOT_SUPPORTED
        HIPSOLVER_STATUS_ARCH_MISMATCH
        HIPSOLVER_STATUS_HANDLE_IS_NULLPTR
        HIPSOLVER_STATUS_INVALID_ENUM
        HIPSOLVER_STATUS_UNKNOWN
        HIPSOLVER_STATUS_ZERO_PIVOT
        HIPSOLVER_STATUS_MATRIX_TYPE_NOT_SUPPORTED

    ctypedef enum hipsolverEigMode_t:
        HIPSOLVER_EIG_MODE_NOVECTOR
        HIPSOLVER_EIG_MODE_VECTOR

    ctypedef enum hipsolverEigType_t:
        HIPSOLVER_EIG_TYPE_1
        HIPSOLVER_EIG_TYPE_2
        HIPSOLVER_EIG_TYPE_3

    ctypedef enum hipsolverEigRange_t:
        HIPSOLVER_EIG_RANGE_ALL
        HIPSOLVER_EIG_RANGE_V
        HIPSOLVER_EIG_RANGE_I

    ctypedef enum hipsolverDeterministicMode_t:
        HIPSOLVER_DETERMINISTIC_RESULTS
        HIPSOLVER_ALLOW_NON_DETERMINISTIC_RESULTS


cdef hipsolverStatus_t hipsolverCreate(void ** handle)



cdef hipsolverStatus_t hipsolverDestroy(void * handle)



cdef hipsolverStatus_t hipsolverSetStream(void * handle,hipStream_t streamId)



cdef hipsolverStatus_t hipsolverGetStream(void * handle,hipStream_t* streamId)



cdef hipsolverStatus_t hipsolverSetDeterministicMode(void * handle,hipsolverDeterministicMode_t mode)



cdef hipsolverStatus_t hipsolverGetDeterministicMode(void * handle,hipsolverDeterministicMode_t * mode)



cdef hipsolverStatus_t hipsolverCreateGesvdjInfo(void ** info)



cdef hipsolverStatus_t hipsolverDestroyGesvdjInfo(void * info)



cdef hipsolverStatus_t hipsolverXgesvdjSetMaxSweeps(void * info,int max_sweeps)



cdef hipsolverStatus_t hipsolverXgesvdjSetSortEig(void * info,int sort_eig)



cdef hipsolverStatus_t hipsolverXgesvdjSetTolerance(void * info,double tolerance)



cdef hipsolverStatus_t hipsolverXgesvdjGetResidual(void * handle,void * info,double * residual)



cdef hipsolverStatus_t hipsolverXgesvdjGetSweeps(void * handle,void * info,int * executed_sweeps)



cdef hipsolverStatus_t hipsolverCreateSyevjInfo(void ** info)



cdef hipsolverStatus_t hipsolverDestroySyevjInfo(void * info)



cdef hipsolverStatus_t hipsolverXsyevjSetMaxSweeps(void * info,int max_sweeps)



cdef hipsolverStatus_t hipsolverXsyevjSetSortEig(void * info,int sort_eig)



cdef hipsolverStatus_t hipsolverXsyevjSetTolerance(void * info,double tolerance)



cdef hipsolverStatus_t hipsolverXsyevjGetResidual(void * handle,void * info,double * residual)



cdef hipsolverStatus_t hipsolverXsyevjGetSweeps(void * handle,void * info,int * executed_sweeps)



cdef hipsolverStatus_t hipsolverSorgbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,float * A,int lda,float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDorgbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,double * A,int lda,double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverCungbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,float2 * A,int lda,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverZungbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,double2 * A,int lda,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverSorgbr(void * handle,hipblasSideMode_t side,int m,int n,int k,float * A,int lda,float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDorgbr(void * handle,hipblasSideMode_t side,int m,int n,int k,double * A,int lda,double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCungbr(void * handle,hipblasSideMode_t side,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZungbr(void * handle,hipblasSideMode_t side,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSorgqr_bufferSize(void * handle,int m,int n,int k,float * A,int lda,float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDorgqr_bufferSize(void * handle,int m,int n,int k,double * A,int lda,double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverCungqr_bufferSize(void * handle,int m,int n,int k,float2 * A,int lda,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverZungqr_bufferSize(void * handle,int m,int n,int k,double2 * A,int lda,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverSorgqr(void * handle,int m,int n,int k,float * A,int lda,float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDorgqr(void * handle,int m,int n,int k,double * A,int lda,double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCungqr(void * handle,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZungqr(void * handle,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSorgtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDorgtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverCungtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverZungtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverSorgtr(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDorgtr(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCungtr(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZungtr(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSormqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,float * A,int lda,float * tau,float * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDormqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,double * A,int lda,double * tau,double * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverCunmqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverZunmqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverSormqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,float * A,int lda,float * tau,float * C,int ldc,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDormqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,double * A,int lda,double * tau,double * C,int ldc,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCunmqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * C,int ldc,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZunmqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * C,int ldc,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSormtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,float * A,int lda,float * tau,float * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDormtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,double * A,int lda,double * tau,double * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverCunmtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,float2 * A,int lda,float2 * tau,float2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverZunmtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,double2 * A,int lda,double2 * tau,double2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverSormtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,float * A,int lda,float * tau,float * C,int ldc,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDormtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,double * A,int lda,double * tau,double * C,int ldc,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCunmtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,float2 * A,int lda,float2 * tau,float2 * C,int ldc,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZunmtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,double2 * A,int lda,double2 * tau,double2 * C,int ldc,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverCgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverZgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverSgebrd(void * handle,int m,int n,float * A,int lda,float * D,float * E,float * tauq,float * taup,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDgebrd(void * handle,int m,int n,double * A,int lda,double * D,double * E,double * tauq,double * taup,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCgebrd(void * handle,int m,int n,float2 * A,int lda,float * D,float * E,float2 * tauq,float2 * taup,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZgebrd(void * handle,int m,int n,double2 * A,int lda,double * D,double * E,double2 * tauq,double2 * taup,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSSgels_bufferSize(void * handle,int m,int n,int nrhs,float * A,int lda,float * B,int ldb,float * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDDgels_bufferSize(void * handle,int m,int n,int nrhs,double * A,int lda,double * B,int ldb,double * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverCCgels_bufferSize(void * handle,int m,int n,int nrhs,float2 * A,int lda,float2 * B,int ldb,float2 * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverZZgels_bufferSize(void * handle,int m,int n,int nrhs,double2 * A,int lda,double2 * B,int ldb,double2 * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverSSgels(void * handle,int m,int n,int nrhs,float * A,int lda,float * B,int ldb,float * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDDgels(void * handle,int m,int n,int nrhs,double * A,int lda,double * B,int ldb,double * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverCCgels(void * handle,int m,int n,int nrhs,float2 * A,int lda,float2 * B,int ldb,float2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverZZgels(void * handle,int m,int n,int nrhs,double2 * A,int lda,double2 * B,int ldb,double2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverSgeqrf_bufferSize(void * handle,int m,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDgeqrf_bufferSize(void * handle,int m,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverCgeqrf_bufferSize(void * handle,int m,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverZgeqrf_bufferSize(void * handle,int m,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverSgeqrf(void * handle,int m,int n,float * A,int lda,float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDgeqrf(void * handle,int m,int n,double * A,int lda,double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCgeqrf(void * handle,int m,int n,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZgeqrf(void * handle,int m,int n,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSSgesv_bufferSize(void * handle,int n,int nrhs,float * A,int lda,int * devIpiv,float * B,int ldb,float * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDDgesv_bufferSize(void * handle,int n,int nrhs,double * A,int lda,int * devIpiv,double * B,int ldb,double * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverCCgesv_bufferSize(void * handle,int n,int nrhs,float2 * A,int lda,int * devIpiv,float2 * B,int ldb,float2 * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverZZgesv_bufferSize(void * handle,int n,int nrhs,double2 * A,int lda,int * devIpiv,double2 * B,int ldb,double2 * X,int ldx,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverSSgesv(void * handle,int n,int nrhs,float * A,int lda,int * devIpiv,float * B,int ldb,float * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDDgesv(void * handle,int n,int nrhs,double * A,int lda,int * devIpiv,double * B,int ldb,double * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverCCgesv(void * handle,int n,int nrhs,float2 * A,int lda,int * devIpiv,float2 * B,int ldb,float2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverZZgesv(void * handle,int n,int nrhs,double2 * A,int lda,int * devIpiv,double2 * B,int ldb,double2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverSgesvd_bufferSize(void * handle,signed char jobu,signed char jobv,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDgesvd_bufferSize(void * handle,signed char jobu,signed char jobv,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverCgesvd_bufferSize(void * handle,signed char jobu,signed char jobv,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverZgesvd_bufferSize(void * handle,signed char jobu,signed char jobv,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverSgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,float * A,int lda,float * S,float * U,int ldu,float * V,int ldv,float * work,int lwork,float * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,double * A,int lda,double * S,double * U,int ldu,double * V,int ldv,double * work,int lwork,double * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,float2 * A,int lda,float * S,float2 * U,int ldu,float2 * V,int ldv,float2 * work,int lwork,float * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,double2 * A,int lda,double * S,double2 * U,int ldu,double2 * V,int ldv,double2 * work,int lwork,double * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,const float * A,int lda,const float * S,const float * U,int ldu,const float * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,const double * A,int lda,const double * S,const double * U,int ldu,const double * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverCgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,float2 * A,int lda,const float * S,float2 * U,int ldu,float2 * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverZgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,double2 * A,int lda,const double * S,double2 * U,int ldu,double2 * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverSgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,float * A,int lda,float * S,float * U,int ldu,float * V,int ldv,float * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,double * A,int lda,double * S,double * U,int ldu,double * V,int ldv,double * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverCgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,float2 * A,int lda,float * S,float2 * U,int ldu,float2 * V,int ldv,float2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverZgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,double2 * A,int lda,double * S,double2 * U,int ldu,double2 * V,int ldv,double2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverSgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,const float * A,int lda,const float * S,const float * U,int ldu,const float * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,const double * A,int lda,const double * S,const double * U,int ldu,const double * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverCgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,float2 * A,int lda,const float * S,float2 * U,int ldu,float2 * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverZgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,double2 * A,int lda,const double * S,double2 * U,int ldu,double2 * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverSgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,float * A,int lda,float * S,float * U,int ldu,float * V,int ldv,float * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,double * A,int lda,double * S,double * U,int ldu,double * V,int ldv,double * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverCgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,float2 * A,int lda,float * S,float2 * U,int ldu,float2 * V,int ldv,float2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverZgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,double2 * A,int lda,double * S,double2 * U,int ldu,double2 * V,int ldv,double2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverSgetrf_bufferSize(void * handle,int m,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDgetrf_bufferSize(void * handle,int m,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverCgetrf_bufferSize(void * handle,int m,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverZgetrf_bufferSize(void * handle,int m,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverSgetrf(void * handle,int m,int n,float * A,int lda,float * work,int lwork,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverDgetrf(void * handle,int m,int n,double * A,int lda,double * work,int lwork,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverCgetrf(void * handle,int m,int n,float2 * A,int lda,float2 * work,int lwork,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverZgetrf(void * handle,int m,int n,double2 * A,int lda,double2 * work,int lwork,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverSgetrs_bufferSize(void * handle,hipblasOperation_t trans,int n,int nrhs,float * A,int lda,int * devIpiv,float * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverDgetrs_bufferSize(void * handle,hipblasOperation_t trans,int n,int nrhs,double * A,int lda,int * devIpiv,double * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverCgetrs_bufferSize(void * handle,hipblasOperation_t trans,int n,int nrhs,float2 * A,int lda,int * devIpiv,float2 * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverZgetrs_bufferSize(void * handle,hipblasOperation_t trans,int n,int nrhs,double2 * A,int lda,int * devIpiv,double2 * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverSgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,float * A,int lda,int * devIpiv,float * B,int ldb,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,double * A,int lda,int * devIpiv,double * B,int ldb,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,float2 * A,int lda,int * devIpiv,float2 * B,int ldb,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,double2 * A,int lda,int * devIpiv,double2 * B,int ldb,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverCpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverZpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverSpotrf(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDpotrf(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCpotrf(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZpotrf(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSpotrfBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float ** A,int lda,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverDpotrfBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double ** A,int lda,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverCpotrfBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 ** A,int lda,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverZpotrfBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 ** A,int lda,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverSpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,float ** A,int lda,float * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,double ** A,int lda,double * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverCpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,float2 ** A,int lda,float2 * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverZpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,double2 ** A,int lda,double2 * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverSpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverCpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverZpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverSpotri(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDpotri(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCpotri(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZpotri(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSpotrs_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float * A,int lda,float * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverDpotrs_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double * A,int lda,double * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverCpotrs_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float2 * A,int lda,float2 * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverZpotrs_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double2 * A,int lda,double2 * B,int ldb,int * lwork)



cdef hipsolverStatus_t hipsolverSpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float * A,int lda,float * B,int ldb,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double * A,int lda,double * B,int ldb,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float2 * A,int lda,float2 * B,int ldb,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double2 * A,int lda,double2 * B,int ldb,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSpotrsBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float ** A,int lda,float ** B,int ldb,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverDpotrsBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double ** A,int lda,double ** B,int ldb,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverCpotrsBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float2 ** A,int lda,float2 ** B,int ldb,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverZpotrsBatched_bufferSize(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double2 ** A,int lda,double2 ** B,int ldb,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverSpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float ** A,int lda,float ** B,int ldb,float * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double ** A,int lda,double ** B,int ldb,double * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverCpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float2 ** A,int lda,float2 ** B,int ldb,float2 * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverZpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double2 ** A,int lda,double2 ** B,int ldb,double2 * work,int lwork,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverSsyevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * D,int * lwork)



cdef hipsolverStatus_t hipsolverDsyevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * D,int * lwork)



cdef hipsolverStatus_t hipsolverCheevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * D,int * lwork)



cdef hipsolverStatus_t hipsolverZheevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * D,int * lwork)



cdef hipsolverStatus_t hipsolverSsyevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * D,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDsyevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * D,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCheevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * D,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZheevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * D,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSsyevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const float * A,int lda,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDsyevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const double * A,int lda,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverCheevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverZheevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverSsyevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float * A,int lda,float vl,float vu,int il,int iu,int * nev,float * W,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDsyevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double * A,int lda,double vl,double vu,int il,int iu,int * nev,double * W,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCheevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float vl,float vu,int il,int iu,int * nev,float * W,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZheevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double vl,double vu,int il,int iu,int * nev,double * W,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSsyevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDsyevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverCheevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverZheevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverSsyevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * W,float * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDsyevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * W,double * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverCheevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * W,float2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverZheevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * W,double2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverSsyevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDsyevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverCheevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverZheevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverSsyevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * W,float * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDsyevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * W,double * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverCheevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * W,float2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverZheevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * W,double2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverSsygvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDsygvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double * W,int * lwork)



cdef hipsolverStatus_t hipsolverChegvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float * W,int * lwork)



cdef hipsolverStatus_t hipsolverZhegvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double * W,int * lwork)



cdef hipsolverStatus_t hipsolverSsygvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float * W,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDsygvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double * W,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverChegvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float * W,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZhegvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double * W,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSsygvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * B,int ldb,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDsygvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * B,int ldb,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverChegvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverZhegvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverSsygvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float vl,float vu,int il,int iu,int * nev,float * W,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDsygvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double vl,double vu,int il,int iu,int * nev,double * W,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverChegvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float vl,float vu,int il,int iu,int * nev,float * W,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZhegvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double vl,double vu,int il,int iu,int * nev,double * W,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSsygvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDsygvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverChegvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverZhegvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverSsygvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float * W,float * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDsygvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double * W,double * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverChegvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float * W,float2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverZhegvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double * W,double2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverSsytrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * D,float * E,float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDsytrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * D,double * E,double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverChetrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * D,float * E,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverZhetrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * D,double * E,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverSsytrd(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * D,float * E,float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDsytrd(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * D,double * E,double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverChetrd(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * D,float * E,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZhetrd(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * D,double * E,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverSsytrf_bufferSize(void * handle,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDsytrf_bufferSize(void * handle,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverCsytrf_bufferSize(void * handle,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverZsytrf_bufferSize(void * handle,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverSsytrf(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,int * ipiv,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDsytrf(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,int * ipiv,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverCsytrf(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,int * ipiv,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverZsytrf(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,int * ipiv,double2 * work,int lwork,int * devInfo)


#   \brief An alias for #hipsolverCreate.
# ******************************************************************************
cdef hipsolverStatus_t hipsolverDnCreate(void ** handle)


#   \brief An alias for #hipsolverDestroy.
# ******************************************************************************
cdef hipsolverStatus_t hipsolverDnDestroy(void * handle)


#   \brief An alias for #hipsolverSetStream.
# ******************************************************************************
cdef hipsolverStatus_t hipsolverDnSetStream(void * handle,hipStream_t streamId)


#   \brief An alias for #hipsolverGetStream.
# ******************************************************************************
cdef hipsolverStatus_t hipsolverDnGetStream(void * handle,hipStream_t* streamId)


#   \brief An alias for #hipsolverSetDeterministicMode.
# ******************************************************************************
cdef hipsolverStatus_t hipsolverDnSetDeterministicMode(void * handle,hipsolverDeterministicMode_t mode)


#   \brief An alias for #hipsolverGetDeterministicMode.
# ******************************************************************************
cdef hipsolverStatus_t hipsolverDnGetDeterministicMode(void * handle,hipsolverDeterministicMode_t * mode)



cdef hipsolverStatus_t hipsolverDnCreateGesvdjInfo(void ** info)



cdef hipsolverStatus_t hipsolverDnDestroyGesvdjInfo(void * info)



cdef hipsolverStatus_t hipsolverDnXgesvdjSetMaxSweeps(void * info,int max_sweeps)



cdef hipsolverStatus_t hipsolverDnXgesvdjSetSortEig(void * info,int sort_eig)



cdef hipsolverStatus_t hipsolverDnXgesvdjSetTolerance(void * info,double tolerance)



cdef hipsolverStatus_t hipsolverDnXgesvdjGetResidual(void * handle,void * info,double * residual)



cdef hipsolverStatus_t hipsolverDnXgesvdjGetSweeps(void * handle,void * info,int * executed_sweeps)



cdef hipsolverStatus_t hipsolverDnCreateSyevjInfo(void ** info)



cdef hipsolverStatus_t hipsolverDnDestroySyevjInfo(void * info)



cdef hipsolverStatus_t hipsolverDnXsyevjSetMaxSweeps(void * info,int max_sweeps)



cdef hipsolverStatus_t hipsolverDnXsyevjSetSortEig(void * info,int sort_eig)



cdef hipsolverStatus_t hipsolverDnXsyevjSetTolerance(void * info,double tolerance)



cdef hipsolverStatus_t hipsolverDnXsyevjGetResidual(void * handle,void * info,double * residual)



cdef hipsolverStatus_t hipsolverDnXsyevjGetSweeps(void * handle,void * info,int * executed_sweeps)



cdef hipsolverStatus_t hipsolverDnSorgbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,const float * A,int lda,const float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnDorgbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,const double * A,int lda,const double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnCungbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,float2 * A,int lda,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnZungbr_bufferSize(void * handle,hipblasSideMode_t side,int m,int n,int k,double2 * A,int lda,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnSorgbr(void * handle,hipblasSideMode_t side,int m,int n,int k,float * A,int lda,const float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDorgbr(void * handle,hipblasSideMode_t side,int m,int n,int k,double * A,int lda,const double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCungbr(void * handle,hipblasSideMode_t side,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZungbr(void * handle,hipblasSideMode_t side,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSorgqr_bufferSize(void * handle,int m,int n,int k,const float * A,int lda,const float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnDorgqr_bufferSize(void * handle,int m,int n,int k,const double * A,int lda,const double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnCungqr_bufferSize(void * handle,int m,int n,int k,float2 * A,int lda,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnZungqr_bufferSize(void * handle,int m,int n,int k,double2 * A,int lda,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnSorgqr(void * handle,int m,int n,int k,float * A,int lda,const float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDorgqr(void * handle,int m,int n,int k,double * A,int lda,const double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCungqr(void * handle,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZungqr(void * handle,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSorgtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnDorgtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnCungtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnZungtr_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnSorgtr(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,const float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDorgtr(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,const double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCungtr(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZungtr(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSormqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,const float * A,int lda,const float * tau,const float * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnDormqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,const double * A,int lda,const double * tau,const double * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnCunmqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnZunmqr_bufferSize(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnSormqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,const float * A,int lda,const float * tau,float * C,int ldc,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDormqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,const double * A,int lda,const double * tau,double * C,int ldc,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCunmqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,float2 * A,int lda,float2 * tau,float2 * C,int ldc,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZunmqr(void * handle,hipblasSideMode_t side,hipblasOperation_t trans,int m,int n,int k,double2 * A,int lda,double2 * tau,double2 * C,int ldc,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSormtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,const float * A,int lda,const float * tau,const float * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnDormtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,const double * A,int lda,const double * tau,const double * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnCunmtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,float2 * A,int lda,float2 * tau,float2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnZunmtr_bufferSize(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,double2 * A,int lda,double2 * tau,double2 * C,int ldc,int * lwork)



cdef hipsolverStatus_t hipsolverDnSormtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,float * A,int lda,float * tau,float * C,int ldc,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDormtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,double * A,int lda,double * tau,double * C,int ldc,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCunmtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,float2 * A,int lda,float2 * tau,float2 * C,int ldc,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZunmtr(void * handle,hipblasSideMode_t side,hipblasFillMode_t uplo,hipblasOperation_t trans,int m,int n,double2 * A,int lda,double2 * tau,double2 * C,int ldc,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnDgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnCgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnZgebrd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnSgebrd(void * handle,int m,int n,float * A,int lda,float * D,float * E,float * tauq,float * taup,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDgebrd(void * handle,int m,int n,double * A,int lda,double * D,double * E,double * tauq,double * taup,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCgebrd(void * handle,int m,int n,float2 * A,int lda,float * D,float * E,float2 * tauq,float2 * taup,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZgebrd(void * handle,int m,int n,double2 * A,int lda,double * D,double * E,double2 * tauq,double2 * taup,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSSgels_bufferSize(void * handle,int m,int n,int nrhs,float * A,int lda,float * B,int ldb,float * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnDDgels_bufferSize(void * handle,int m,int n,int nrhs,double * A,int lda,double * B,int ldb,double * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnCCgels_bufferSize(void * handle,int m,int n,int nrhs,float2 * A,int lda,float2 * B,int ldb,float2 * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnZZgels_bufferSize(void * handle,int m,int n,int nrhs,double2 * A,int lda,double2 * B,int ldb,double2 * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnSSgels(void * handle,int m,int n,int nrhs,float * A,int lda,float * B,int ldb,float * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDDgels(void * handle,int m,int n,int nrhs,double * A,int lda,double * B,int ldb,double * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCCgels(void * handle,int m,int n,int nrhs,float2 * A,int lda,float2 * B,int ldb,float2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZZgels(void * handle,int m,int n,int nrhs,double2 * A,int lda,double2 * B,int ldb,double2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSgeqrf_bufferSize(void * handle,int m,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnDgeqrf_bufferSize(void * handle,int m,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnCgeqrf_bufferSize(void * handle,int m,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnZgeqrf_bufferSize(void * handle,int m,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnSgeqrf(void * handle,int m,int n,float * A,int lda,float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDgeqrf(void * handle,int m,int n,double * A,int lda,double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCgeqrf(void * handle,int m,int n,float2 * A,int lda,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZgeqrf(void * handle,int m,int n,double2 * A,int lda,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSSgesv_bufferSize(void * handle,int n,int nrhs,float * A,int lda,int * devIpiv,float * B,int ldb,float * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnDDgesv_bufferSize(void * handle,int n,int nrhs,double * A,int lda,int * devIpiv,double * B,int ldb,double * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnCCgesv_bufferSize(void * handle,int n,int nrhs,float2 * A,int lda,int * devIpiv,float2 * B,int ldb,float2 * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnZZgesv_bufferSize(void * handle,int n,int nrhs,double2 * A,int lda,int * devIpiv,double2 * B,int ldb,double2 * X,int ldx,void * work,unsigned long * lwork)



cdef hipsolverStatus_t hipsolverDnSSgesv(void * handle,int n,int nrhs,float * A,int lda,int * devIpiv,float * B,int ldb,float * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDDgesv(void * handle,int n,int nrhs,double * A,int lda,int * devIpiv,double * B,int ldb,double * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCCgesv(void * handle,int n,int nrhs,float2 * A,int lda,int * devIpiv,float2 * B,int ldb,float2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZZgesv(void * handle,int n,int nrhs,double2 * A,int lda,int * devIpiv,double2 * B,int ldb,double2 * X,int ldx,void * work,unsigned long lwork,int * niters,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSgesvd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnDgesvd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnCgesvd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnZgesvd_bufferSize(void * handle,int m,int n,int * lwork)



cdef hipsolverStatus_t hipsolverDnSgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,float * A,int lda,float * S,float * U,int ldu,float * V,int ldv,float * work,int lwork,float * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,double * A,int lda,double * S,double * U,int ldu,double * V,int ldv,double * work,int lwork,double * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,float2 * A,int lda,float * S,float2 * U,int ldu,float2 * V,int ldv,float2 * work,int lwork,float * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZgesvd(void * handle,signed char jobu,signed char jobv,int m,int n,double2 * A,int lda,double * S,double2 * U,int ldu,double2 * V,int ldv,double2 * work,int lwork,double * rwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,const float * A,int lda,const float * S,const float * U,int ldu,const float * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnDgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,const double * A,int lda,const double * S,const double * U,int ldu,const double * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnCgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,float2 * A,int lda,const float * S,float2 * U,int ldu,float2 * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnZgesvdj_bufferSize(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,double2 * A,int lda,const double * S,double2 * U,int ldu,double2 * V,int ldv,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnSgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,float * A,int lda,float * S,float * U,int ldu,float * V,int ldv,float * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnDgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,double * A,int lda,double * S,double * U,int ldu,double * V,int ldv,double * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnCgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,float2 * A,int lda,float * S,float2 * U,int ldu,float2 * V,int ldv,float2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnZgesvdj(void * handle,hipsolverEigMode_t jobz,int econ,int m,int n,double2 * A,int lda,double * S,double2 * U,int ldu,double2 * V,int ldv,double2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnSgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,const float * A,int lda,const float * S,const float * U,int ldu,const float * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnDgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,const double * A,int lda,const double * S,const double * U,int ldu,const double * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnCgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,float2 * A,int lda,const float * S,float2 * U,int ldu,float2 * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnZgesvdjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int m,int n,double2 * A,int lda,const double * S,double2 * U,int ldu,double2 * V,int ldv,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnSgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,float * A,int lda,float * S,float * U,int ldu,float * V,int ldv,float * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnDgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,double * A,int lda,double * S,double * U,int ldu,double * V,int ldv,double * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnCgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,float2 * A,int lda,float * S,float2 * U,int ldu,float2 * V,int ldv,float2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnZgesvdjBatched(void * handle,hipsolverEigMode_t jobz,int m,int n,double2 * A,int lda,double * S,double2 * U,int ldu,double2 * V,int ldv,double2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnSgesvdaStridedBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,const float * A,int lda,long long strideA,const float * S,long long strideS,const float * U,int ldu,long long strideU,const float * V,int ldv,long long strideV,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverDnDgesvdaStridedBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,const double * A,int lda,long long strideA,const double * S,long long strideS,const double * U,int ldu,long long strideU,const double * V,int ldv,long long strideV,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverDnCgesvdaStridedBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,float2 * A,int lda,long long strideA,const float * S,long long strideS,float2 * U,int ldu,long long strideU,float2 * V,int ldv,long long strideV,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverDnZgesvdaStridedBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,double2 * A,int lda,long long strideA,const double * S,long long strideS,double2 * U,int ldu,long long strideU,double2 * V,int ldv,long long strideV,int * lwork,int batch_count)



cdef hipsolverStatus_t hipsolverDnSgesvdaStridedBatched(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,const float * A,int lda,long long strideA,float * S,long long strideS,float * U,int ldu,long long strideU,float * V,int ldv,long long strideV,float * work,int lwork,int * devInfo,double * hRnrmF,int batch_count)



cdef hipsolverStatus_t hipsolverDnDgesvdaStridedBatched(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,const double * A,int lda,long long strideA,double * S,long long strideS,double * U,int ldu,long long strideU,double * V,int ldv,long long strideV,double * work,int lwork,int * devInfo,double * hRnrmF,int batch_count)



cdef hipsolverStatus_t hipsolverDnCgesvdaStridedBatched(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,float2 * A,int lda,long long strideA,float * S,long long strideS,float2 * U,int ldu,long long strideU,float2 * V,int ldv,long long strideV,float2 * work,int lwork,int * devInfo,double * hRnrmF,int batch_count)



cdef hipsolverStatus_t hipsolverDnZgesvdaStridedBatched(void * handle,hipsolverEigMode_t jobz,int rank,int m,int n,double2 * A,int lda,long long strideA,double * S,long long strideS,double2 * U,int ldu,long long strideU,double2 * V,int ldv,long long strideV,double2 * work,int lwork,int * devInfo,double * hRnrmF,int batch_count)



cdef hipsolverStatus_t hipsolverDnSgetrf_bufferSize(void * handle,int m,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnDgetrf_bufferSize(void * handle,int m,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnCgetrf_bufferSize(void * handle,int m,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnZgetrf_bufferSize(void * handle,int m,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnSgetrf(void * handle,int m,int n,float * A,int lda,float * work,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDgetrf(void * handle,int m,int n,double * A,int lda,double * work,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCgetrf(void * handle,int m,int n,float2 * A,int lda,float2 * work,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZgetrf(void * handle,int m,int n,double2 * A,int lda,double2 * work,int * devIpiv,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,const float * A,int lda,const int * devIpiv,float * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,const double * A,int lda,const int * devIpiv,double * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,float2 * A,int lda,const int * devIpiv,float2 * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZgetrs(void * handle,hipblasOperation_t trans,int n,int nrhs,double2 * A,int lda,const int * devIpiv,double2 * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnDpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnCpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnZpotrf_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnSpotrf(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDpotrf(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCpotrf(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZpotrf(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,float ** A,int lda,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnDpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,double ** A,int lda,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnCpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,float2 ** A,int lda,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnZpotrfBatched(void * handle,hipblasFillMode_t uplo,int n,double2 ** A,int lda,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnSpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnDpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnCpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnZpotri_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnSpotri(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDpotri(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCpotri(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZpotri(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,const float * A,int lda,float * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,const double * A,int lda,double * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float2 * A,int lda,float2 * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZpotrs(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double2 * A,int lda,double2 * B,int ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float ** A,int lda,float ** B,int ldb,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnDpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double ** A,int lda,double ** B,int ldb,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnCpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,float2 ** A,int lda,float2 ** B,int ldb,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnZpotrsBatched(void * handle,hipblasFillMode_t uplo,int n,int nrhs,double2 ** A,int lda,double2 ** B,int ldb,int * devInfo,int batch_count)



cdef hipsolverStatus_t hipsolverDnSsyevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnDsyevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnCheevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnZheevd_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnSsyevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * W,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDsyevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * W,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCheevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * W,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZheevd(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * W,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSsyevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const float * A,int lda,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnDsyevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const double * A,int lda,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnCheevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnZheevdx_bufferSize(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnSsyevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float * A,int lda,float vl,float vu,int il,int iu,int * nev,float * W,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDsyevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double * A,int lda,double vl,double vu,int il,int iu,int * nev,double * W,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCheevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float vl,float vu,int il,int iu,int * nev,float * W,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZheevdx(void * handle,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double vl,double vu,int il,int iu,int * nev,double * W,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSsyevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnDsyevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnCheevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,const float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnZheevj_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,const double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnSsyevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * W,float * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnDsyevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * W,double * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnCheevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * W,float2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnZheevj(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * W,double2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnSsyevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnDsyevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnCheevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,const float * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnZheevjBatched_bufferSize(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,const double * W,int * lwork,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnSsyevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * W,float * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnDsyevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * W,double * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnCheevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * W,float2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnZheevjBatched(void * handle,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * W,double2 * work,int lwork,int * devInfo,void * params,int batch_count)



cdef hipsolverStatus_t hipsolverDnSsygvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * B,int ldb,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnDsygvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * B,int ldb,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnChegvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnZhegvd_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnSsygvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float * W,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDsygvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double * W,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnChegvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float * W,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZhegvd(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double * W,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSsygvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * B,int ldb,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnDsygvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * B,int ldb,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnChegvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float vl,float vu,int il,int iu,int * nev,const float * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnZhegvdx_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double vl,double vu,int il,int iu,int * nev,const double * W,int * lwork)



cdef hipsolverStatus_t hipsolverDnSsygvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float vl,float vu,int il,int iu,int * nev,float * W,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDsygvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double vl,double vu,int il,int iu,int * nev,double * W,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnChegvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float vl,float vu,int il,int iu,int * nev,float * W,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZhegvdx(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipsolverEigRange_t range,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double vl,double vu,int il,int iu,int * nev,double * W,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSsygvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * B,int ldb,const float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnDsygvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * B,int ldb,const double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnChegvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,const float * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnZhegvj_bufferSize(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,const double * W,int * lwork,void * params)



cdef hipsolverStatus_t hipsolverDnSsygvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float * A,int lda,float * B,int ldb,float * W,float * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnDsygvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double * A,int lda,double * B,int ldb,double * W,double * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnChegvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,float2 * A,int lda,float2 * B,int ldb,float * W,float2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnZhegvj(void * handle,hipsolverEigType_t itype,hipsolverEigMode_t jobz,hipblasFillMode_t uplo,int n,double2 * A,int lda,double2 * B,int ldb,double * W,double2 * work,int lwork,int * devInfo,void * params)



cdef hipsolverStatus_t hipsolverDnSsytrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,const float * A,int lda,const float * D,const float * E,const float * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnDsytrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,const double * A,int lda,const double * D,const double * E,const double * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnChetrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,const float * D,const float * E,float2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnZhetrd_bufferSize(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,const double * D,const double * E,double2 * tau,int * lwork)



cdef hipsolverStatus_t hipsolverDnSsytrd(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,float * D,float * E,float * tau,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDsytrd(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,double * D,double * E,double * tau,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnChetrd(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,float * D,float * E,float2 * tau,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZhetrd(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,double * D,double * E,double2 * tau,double2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnSsytrf_bufferSize(void * handle,int n,float * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnDsytrf_bufferSize(void * handle,int n,double * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnCsytrf_bufferSize(void * handle,int n,float2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnZsytrf_bufferSize(void * handle,int n,double2 * A,int lda,int * lwork)



cdef hipsolverStatus_t hipsolverDnSsytrf(void * handle,hipblasFillMode_t uplo,int n,float * A,int lda,int * ipiv,float * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnDsytrf(void * handle,hipblasFillMode_t uplo,int n,double * A,int lda,int * ipiv,double * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnCsytrf(void * handle,hipblasFillMode_t uplo,int n,float2 * A,int lda,int * ipiv,float2 * work,int lwork,int * devInfo)



cdef hipsolverStatus_t hipsolverDnZsytrf(void * handle,hipblasFillMode_t uplo,int n,double2 * A,int lda,int * ipiv,double2 * work,int lwork,int * devInfo)


cdef extern from "hipsolver/hipsolver.h":

    ctypedef void * hipsolverDnParams_t

    ctypedef enum hipsolverAlgMode_t:
        HIPSOLVER_ALG_0
        HIPSOLVER_ALG_1

    ctypedef enum hipsolverDnFunction_t:
        HIPSOLVERDN_GETRF


cdef hipsolverStatus_t hipsolverDnCreateParams(void ** params)



cdef hipsolverStatus_t hipsolverDnDestroyParams(void * params)



cdef hipsolverStatus_t hipsolverDnSetAdvOptions(void * params,hipsolverDnFunction_t func,hipsolverAlgMode_t alg)



cdef hipsolverStatus_t hipsolverDnXgeqrf_bufferSize(void * handle,void * params,long m,long n,hipDataType dataTypeA,const void * A,long lda,hipDataType dataTypeTau,const void * tau,hipDataType computeType,unsigned long * lworkOnDevice,unsigned long * lworkOnHost)



cdef hipsolverStatus_t hipsolverDnXgeqrf(void * handle,void * params,long m,long n,hipDataType dataTypeA,void * A,long lda,hipDataType dataTypeTau,void * tau,hipDataType computeType,void * workOnDevice,unsigned long lworkOnDevice,void * workOnHost,unsigned long lworkOnHost,int * devInfo)



cdef hipsolverStatus_t hipsolverDnXgetrf_bufferSize(void * handle,void * params,long m,long n,hipDataType dataTypeA,const void * A,long lda,hipDataType computeType,unsigned long * lworkOnDevice,unsigned long * lworkOnHost)



cdef hipsolverStatus_t hipsolverDnXgetrf(void * handle,void * params,long m,long n,hipDataType dataTypeA,void * A,long lda,long * devIpiv,hipDataType computeType,void * workOnDevice,unsigned long lworkOnDevice,void * workOnHost,unsigned long lworkOnHost,int * devInfo)



cdef hipsolverStatus_t hipsolverDnXgetrs(void * handle,void * params,hipblasOperation_t trans,long n,long nrhs,hipDataType dataTypeA,const void * A,long lda,const long * devIpiv,hipDataType dataTypeB,void * B,long ldb,int * devInfo)



cdef hipsolverStatus_t hipsolverDnXpotrf_bufferSize(void * handle,void * params,hipblasFillMode_t uplo,long n,hipDataType dataTypeA,const void * A,long lda,hipDataType computeType,unsigned long * lworkOnDevice,unsigned long * lworkOnHost)



cdef hipsolverStatus_t hipsolverDnXpotrf(void * handle,void * params,hipblasFillMode_t uplo,long n,hipDataType dataTypeA,void * A,long lda,hipDataType computeType,void * workOnDevice,unsigned long lworkOnDevice,void * workOnHost,unsigned long lworkOnHost,int * info)



cdef hipsolverStatus_t hipsolverDnXpotrs(void * handle,void * params,hipblasFillMode_t uplo,long n,long nrhs,hipDataType dataTypeA,const void * A,long lda,hipDataType dataTypeB,void * B,long ldb,int * info)


cdef extern from "hipsolver/hipsolver.h":

    ctypedef void * hipsolverRfHandle_t

    ctypedef enum hipsolverRfFactorization_t:
        HIPSOLVERRF_FACTORIZATION_ALG0
        HIPSOLVERRF_FACTORIZATION_ALG1
        HIPSOLVERRF_FACTORIZATION_ALG2

    ctypedef enum hipsolverRfMatrixFormat_t:
        HIPSOLVERRF_MATRIX_FORMAT_CSR
        HIPSOLVERRF_MATRIX_FORMAT_CSC

    ctypedef enum hipsolverRfNumericBoostReport_t:
        HIPSOLVERRF_NUMERIC_BOOST_NOT_USED
        HIPSOLVERRF_NUMERIC_BOOST_USED

    ctypedef enum hipsolverRfResetValuesFastMode_t:
        HIPSOLVERRF_RESET_VALUES_FAST_MODE_OFF
        HIPSOLVERRF_RESET_VALUES_FAST_MODE_ON

    ctypedef enum hipsolverRfTriangularSolve_t:
        HIPSOLVERRF_TRIANGULAR_SOLVE_ALG1
        HIPSOLVERRF_TRIANGULAR_SOLVE_ALG2
        HIPSOLVERRF_TRIANGULAR_SOLVE_ALG3

    ctypedef enum hipsolverRfUnitDiagonal_t:
        HIPSOLVERRF_UNIT_DIAGONAL_STORED_L
        HIPSOLVERRF_UNIT_DIAGONAL_STORED_U
        HIPSOLVERRF_UNIT_DIAGONAL_ASSUMED_L
        HIPSOLVERRF_UNIT_DIAGONAL_ASSUMED_U


cdef hipsolverStatus_t hipsolverRfCreate(void ** handle)



cdef hipsolverStatus_t hipsolverRfDestroy(void * handle)



cdef hipsolverStatus_t hipsolverRfSetupDevice(int n,int nnzA,int * csrRowPtrA,int * csrColIndA,double * csrValA,int nnzL,int * csrRowPtrL,int * csrColIndL,double * csrValL,int nnzU,int * csrRowPtrU,int * csrColIndU,double * csrValU,int * P,int * Q,void * handle)



cdef hipsolverStatus_t hipsolverRfSetupHost(int n,int nnzA,int * h_csrRowPtrA,int * h_csrColIndA,double * h_csrValA,int nnzL,int * h_csrRowPtrL,int * h_csrColIndL,double * h_csrValL,int nnzU,int * h_csrRowPtrU,int * h_csrColIndU,double * h_csrValU,int * h_P,int * h_Q,void * handle)



cdef hipsolverStatus_t hipsolverRfAccessBundledFactorsDevice(void * handle,int * nnzM,int ** Mp,int ** Mi,double ** Mx)



cdef hipsolverStatus_t hipsolverRfAnalyze(void * handle)



cdef hipsolverStatus_t hipsolverRfExtractBundledFactorsHost(void * handle,int * h_nnzM,int ** h_Mp,int ** h_Mi,double ** h_Mx)



cdef hipsolverStatus_t hipsolverRfExtractSplitFactorsHost(void * handle,int * h_nnzL,int ** h_Lp,int ** h_Li,double ** h_Lx,int * h_nnzU,int ** h_Up,int ** h_Ui,double ** h_Ux)



cdef hipsolverStatus_t hipsolverRfGet_Algs(void * handle,hipsolverRfFactorization_t * fact_alg,hipsolverRfTriangularSolve_t * solve_alg)



cdef hipsolverStatus_t hipsolverRfGetMatrixFormat(void * handle,hipsolverRfMatrixFormat_t * format,hipsolverRfUnitDiagonal_t * diag)



cdef hipsolverStatus_t hipsolverRfGetNumericBoostReport(void * handle,hipsolverRfNumericBoostReport_t * report)



cdef hipsolverStatus_t hipsolverRfGetNumericProperties(void * handle,double * zero,double * boost)



cdef hipsolverStatus_t hipsolverRfGetResetValuesFastMode(void * handle,hipsolverRfResetValuesFastMode_t * fastMode)



cdef hipsolverStatus_t hipsolverRfRefactor(void * handle)



cdef hipsolverStatus_t hipsolverRfResetValues(int n,int nnzA,int * csrRowPtrA,int * csrColIndA,double * csrValA,int * P,int * Q,void * handle)



cdef hipsolverStatus_t hipsolverRfSetAlgs(void * handle,hipsolverRfFactorization_t fact_alg,hipsolverRfTriangularSolve_t solve_alg)



cdef hipsolverStatus_t hipsolverRfSetMatrixFormat(void * handle,hipsolverRfMatrixFormat_t format,hipsolverRfUnitDiagonal_t diag)



cdef hipsolverStatus_t hipsolverRfSetNumericProperties(void * handle,double effective_zero,double boost_val)



cdef hipsolverStatus_t hipsolverRfSetResetValuesFastMode(void * handle,hipsolverRfResetValuesFastMode_t fastMode)



cdef hipsolverStatus_t hipsolverRfSolve(void * handle,int * P,int * Q,int nrhs,double * Temp,int ldt,double * XF,int ldxf)



cdef hipsolverStatus_t hipsolverRfBatchSetupHost(int batchSize,int n,int nnzA,int * h_csrRowPtrA,int * h_csrColIndA,double ** h_csrValA_array,int nnzL,int * h_csrRowPtrL,int * h_csrColIndL,double * h_csrValL,int nnzU,int * h_csrRowPtrU,int * h_csrColIndU,double * h_csrValU,int * h_P,int * h_Q,void * handle)



cdef hipsolverStatus_t hipsolverRfBatchAnalyze(void * handle)



cdef hipsolverStatus_t hipsolverRfBatchRefactor(void * handle)



cdef hipsolverStatus_t hipsolverRfBatchResetValues(int batchSize,int n,int nnzA,int * csrRowPtrA,int * csrColIndA,double ** csrValA_array,int * P,int * Q,void * handle)



cdef hipsolverStatus_t hipsolverRfBatchSolve(void * handle,int * P,int * Q,int nrhs,double * Temp,int ldt,double ** XF_array,int ldxf)



cdef hipsolverStatus_t hipsolverRfBatchZeroPivot(void * handle,int * position)


cdef extern from "hipsolver/hipsolver.h":

    ctypedef void * hipsolverSpHandle_t


cdef hipsolverStatus_t hipsolverSpCreate(void ** handle)



cdef hipsolverStatus_t hipsolverSpDestroy(void * handle)



cdef hipsolverStatus_t hipsolverSpSetStream(void * handle,hipStream_t streamId)



cdef hipsolverStatus_t hipsolverSpScsrlsvchol(void * handle,int n,int nnzA,void *const descrA,const float * csrVal,const int * csrRowPtr,const int * csrColInd,const float * b,float tolerance,int reorder,float * x,int * singularity)



cdef hipsolverStatus_t hipsolverSpDcsrlsvchol(void * handle,int n,int nnzA,void *const descrA,const double * csrVal,const int * csrRowPtr,const int * csrColInd,const double * b,double tolerance,int reorder,double * x,int * singularity)



cdef hipsolverStatus_t hipsolverSpScsrlsvcholHost(void * handle,int n,int nnzA,void *const descrA,const float * csrVal,const int * csrRowPtr,const int * csrColInd,const float * b,float tolerance,int reorder,float * x,int * singularity)



cdef hipsolverStatus_t hipsolverSpDcsrlsvcholHost(void * handle,int n,int nnzA,void *const descrA,const double * csrVal,const int * csrRowPtr,const int * csrColInd,const double * b,double tolerance,int reorder,double * x,int * singularity)
