# MIT License
#
# Copyright (c) 2021-2025 Advanced Micro Devices, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


# This file has been autogenerated, do not modify.


"""
(No short description)

Attributes:
    NCCL_MAJOR (`~.int`):
        Macro constant.

    NCCL_MINOR (`~.int`):
        Macro constant.

    NCCL_PATCH (`~.int`):
        Macro constant.

    NCCL_SUFFIX (`~.bytes`):
        Macro constant.

    NCCL_VERSION_CODE (`~.int`):
        Macro constant.

    RCCL_BFLOAT16 (`~.int`):
        Macro constant.

    RCCL_GATHER_SCATTER (`~.int`):
        Macro constant.

    RCCL_ALLTOALLV (`~.int`):
        Macro constant.

    NCCL_UNIQUE_ID_BYTES (`~.int`):
        Macro constant.

    ncclComm_t:
        alias of `~.ncclComm`

    ncclConfig_t:
        alias of `~.ncclConfig_v21700`

"""

import cython
import ctypes
import enum
NCCL_MAJOR = crccl.NCCL_MAJOR

NCCL_MINOR = crccl.NCCL_MINOR

NCCL_PATCH = crccl.NCCL_PATCH

NCCL_SUFFIX = crccl.NCCL_SUFFIX

NCCL_VERSION_CODE = crccl.NCCL_VERSION_CODE

RCCL_BFLOAT16 = crccl.RCCL_BFLOAT16

RCCL_GATHER_SCATTER = crccl.RCCL_GATHER_SCATTER

RCCL_ALLTOALLV = crccl.RCCL_ALLTOALLV

NCCL_UNIQUE_ID_BYTES = crccl.NCCL_UNIQUE_ID_BYTES

cdef class ncclComm(hip._util.types.Pointer):
    """Python wrapper for cdef class crccl.ncclComm.

    Python wrapper for cdef class crccl.ncclComm.

    If this type is initialized via its `__init__` method, it allocates a
    member of the underlying C type and destroys it again if the wrapper
    type is deallocated.

    This type also serves as adapter when appearing as argument type in a
    function signature. In this case, the type can further be initialized
    from a number of Python objects:

    * `None`:

      This will set the ``self._ptr`` attribute to ``NULL``.

    * `int`:

      Interprets the integer value as pointer address and writes it to ``self._ptr``.
      No ownership is transferred.

    * `ctypes.c_void_p`:

      Takes the pointer address ``pyobj.value`` and writes it to ``self._ptr``.
      No ownership is transferred.

    * `object` that implements the `CUDA Array Interface <https://numba.readthedocs.io/en/stable/cuda/cuda_array_interface.html>`_ protocol:

      Takes the integer-valued pointer address, i.e. the first entry of the `data` tuple
      from `pyobj`'s member ``__cuda_array_interface__``  and writes it to ``self._ptr``.

    * `object` that implements the Python buffer protocol:

      If the object represents a simple contiguous array,
      writes the `Py_buffer` associated with ``pyobj`` to `self._py_buffer`,
      sets the `self._py_buffer_acquired` flag to `True`, and
      writes `self._py_buffer.buf` to the data pointer `self._ptr`.

    * `hip._util.types.Pointer`:

      Takes the pointer address ``pyobj._ptr`` and writes it to ``self._ptr``.
      No ownership is transferred.

    Type checks are performed in the above order.

    C Attributes:
        _ptr (C type ``void *``, protected):
            Stores a pointer to the data of the original Python object.
        _is_ptr_owner (C type ``bint``, protected):
            If this wrapper is the owner of the underlying data.
        _py_buffer (C type ``Py_buffer`, protected):
            Stores a pointer to the data of the original Python object.
        _py_buffer_acquired (C type ``bint``, protected):
            Stores a pointer to the data of the original Python object.
    """
    # C members declared in pxd file

    def __cinit__(self):
        self._ptr = NULL
        self._is_ptr_owner = False
        self._py_buffer_acquired = False

    cdef crccl.ncclComm* getElementPtr(self):
        return <crccl.ncclComm*>self._ptr

    @staticmethod
    cdef ncclComm fromPtr(void* ptr, bint owner=False):
        """Factory function to create ``ncclComm`` objects from
        given ``crccl.ncclComm`` pointer.
        """
        # Fast call to __new__() that bypasses the __init__() constructor.
        cdef ncclComm wrapper = ncclComm.__new__(ncclComm)
        wrapper._ptr = ptr
        wrapper._is_ptr_owner = owner
        return wrapper

    @staticmethod
    def fromObj(pyobj):
        """Creates a ncclComm from a Python object.

        Derives a ncclComm from the given Python object ``pyobj``.
        In case ``pyobj`` is itself an ``ncclComm`` reference, this method
        returns it directly. No new ``ncclComm`` is created in this case.
        """
        return ncclComm.fromPyobj(pyobj)

    @staticmethod
    cdef ncclComm fromPyobj(object pyobj):
        """Creates a ncclComm from a Python object.

        Derives a ncclComm from the given Python object ``pyobj``.
        In case ``pyobj`` is itself an ``ncclComm`` reference, this method
        returns it directly. No new ``ncclComm`` is created in this case.

        Args:
            pyobj (object): Must be either `None`; a `hip._util.types.Pointer`; a simple, contiguous buffer according to the buffer protocol;
                            or of type `ncclComm`; `int`; or `ctypes.c_void_p`.

        Note:
            This routine does not perform a copy but returns the original ``pyobj``
            if ``pyobj`` is an instance of ncclComm!
        """
        cdef ncclComm wrapper

        if isinstance(pyobj,ncclComm):
            return pyobj
        else:
            wrapper = ncclComm.__new__(ncclComm)
            wrapper.init_from_pyobj(pyobj)
            return wrapper

    def __dealloc__(self):
        # Release the buffer handle
        if self._py_buffer_acquired is True:
            cpython.buffer.PyBuffer_Release(&self._py_buffer)

    def __int__(self):
        """Returns the data's address as long integer.
        """
        return cpython.long.PyLong_FromVoidPtr(self._ptr)

    def __repr__(self):
        return f"<ncclComm object, ptr: {int(self)}>"

    def as_c_void_p(self):
        """Returns the data's address as `ctypes.c_void_p`
        Note:
            Implemented as function to not collide with
            autogenerated property names.
        """
        return ctypes.c_void_p(int(self))

    @staticmethod
    def PROPERTIES():
        return []

    def __contains__(self,item):
        properties = self.PROPERTIES()
        return item in properties

    def __getitem__(self,item):
        properties = self.PROPERTIES()
        if isinstance(item,int):
            if item < 0 or item >= len(properties):
                raise IndexError()
            return getattr(self,properties[item])
        raise ValueError("'item' type must be 'int'")


ncclComm_t = ncclComm

cdef class ncclUniqueId(hip._util.types.Pointer):
    """Python wrapper for cdef class crccl.ncclUniqueId.

    Python wrapper for cdef class crccl.ncclUniqueId.

    If this type is initialized via its `__init__` method, it allocates a
    member of the underlying C type and destroys it again if the wrapper
    type is deallocated.

    This type also serves as adapter when appearing as argument type in a
    function signature. In this case, the type can further be initialized
    from a number of Python objects:

    * `None`:

      This will set the ``self._ptr`` attribute to ``NULL``.

    * `int`:

      Interprets the integer value as pointer address and writes it to ``self._ptr``.
      No ownership is transferred.

    * `ctypes.c_void_p`:

      Takes the pointer address ``pyobj.value`` and writes it to ``self._ptr``.
      No ownership is transferred.

    * `object` that implements the `CUDA Array Interface <https://numba.readthedocs.io/en/stable/cuda/cuda_array_interface.html>`_ protocol:

      Takes the integer-valued pointer address, i.e. the first entry of the `data` tuple
      from `pyobj`'s member ``__cuda_array_interface__``  and writes it to ``self._ptr``.

    * `object` that implements the Python buffer protocol:

      If the object represents a simple contiguous array,
      writes the `Py_buffer` associated with ``pyobj`` to `self._py_buffer`,
      sets the `self._py_buffer_acquired` flag to `True`, and
      writes `self._py_buffer.buf` to the data pointer `self._ptr`.

    * `hip._util.types.Pointer`:

      Takes the pointer address ``pyobj._ptr`` and writes it to ``self._ptr``.
      No ownership is transferred.

    Type checks are performed in the above order.

    C Attributes:
        _ptr (C type ``void *``, protected):
            Stores a pointer to the data of the original Python object.
        _is_ptr_owner (C type ``bint``, protected):
            If this wrapper is the owner of the underlying data.
        _py_buffer (C type ``Py_buffer`, protected):
            Stores a pointer to the data of the original Python object.
        _py_buffer_acquired (C type ``bint``, protected):
            Stores a pointer to the data of the original Python object.
    """
    # C members declared in pxd file

    def __cinit__(self):
        self._ptr = NULL
        self._is_ptr_owner = False
        self._py_buffer_acquired = False

    cdef crccl.ncclUniqueId* getElementPtr(self):
        return <crccl.ncclUniqueId*>self._ptr

    @staticmethod
    cdef ncclUniqueId fromPtr(void* ptr, bint owner=False):
        """Factory function to create ``ncclUniqueId`` objects from
        given ``crccl.ncclUniqueId`` pointer.

        Setting ``owner`` flag to ``True`` causes
        the extension type to free the structure pointed to by ``ptr``
        when the wrapper object is deallocated.
        """
        # Fast call to __new__() that bypasses the __init__() constructor.
        cdef ncclUniqueId wrapper = ncclUniqueId.__new__(ncclUniqueId)
        wrapper._ptr = ptr
        wrapper._is_ptr_owner = owner
        return wrapper

    @staticmethod
    def fromObj(pyobj):
        """Creates a ncclUniqueId from a Python object.

        Derives a ncclUniqueId from the given Python object ``pyobj``.
        In case ``pyobj`` is itself an ``ncclUniqueId`` reference, this method
        returns it directly. No new ``ncclUniqueId`` is created in this case.
        """
        return ncclUniqueId.fromPyobj(pyobj)

    @staticmethod
    cdef ncclUniqueId fromPyobj(object pyobj):
        """Creates a ncclUniqueId from a Python object.

        Derives a ncclUniqueId from the given Python object ``pyobj``.
        In case ``pyobj`` is itself an ``ncclUniqueId`` reference, this method
        returns it directly. No new ``ncclUniqueId`` is created in this case.

        Args:
            pyobj (object): Must be either `None`; a `hip._util.types.Pointer`; a simple, contiguous buffer according to the buffer protocol;
                            or of type `ncclUniqueId`; `int`; or `ctypes.c_void_p`.

        Note:
            This routine does not perform a copy but returns the original ``pyobj``
            if ``pyobj`` is an instance of ncclUniqueId!
        """
        cdef ncclUniqueId wrapper

        if isinstance(pyobj,ncclUniqueId):
            return pyobj
        else:
            wrapper = ncclUniqueId.__new__(ncclUniqueId)
            wrapper.init_from_pyobj(pyobj)
            return wrapper

    def __dealloc__(self):
        # Release the buffer handle
        if self._py_buffer_acquired is True:
            cpython.buffer.PyBuffer_Release(&self._py_buffer)
        # De-allocate if not null and flag is set
        if self._ptr is not NULL and self._is_ptr_owner is True:
            stdlib.free(self._ptr)
            self._ptr = NULL

    @staticmethod
    cdef __allocate(void** ptr):
        ptr[0] = stdlib.malloc(sizeof(crccl.ncclUniqueId))
        string.memset(<void*>ptr[0], 0, sizeof(crccl.ncclUniqueId))

        if ptr[0] is NULL:
            raise MemoryError

    @staticmethod
    cdef ncclUniqueId new():
        """Factory function to create ncclUniqueId objects with
        newly allocated crccl.ncclUniqueId"""
        cdef void* ptr
        ncclUniqueId.__allocate(&ptr)
        return ncclUniqueId.fromPtr(ptr, owner=True)

    @staticmethod
    cdef ncclUniqueId fromValue(crccl.ncclUniqueId other):
        """Allocate new C type and copy from ``other``.
        """
        wrapper = ncclUniqueId.new()
        string.memcpy(wrapper._ptr, &other, sizeof(crccl.ncclUniqueId))
        return wrapper

    def c_sizeof(self):
        """Returns the size of the underlying C type in bytes.
        Note:
            Implemented as function to not collide with
            autogenerated property names.
        """
        return sizeof(crccl.ncclUniqueId)

    def __int__(self):
        """Returns the data's address as long integer.
        """
        return cpython.long.PyLong_FromVoidPtr(self._ptr)

    def __repr__(self):
        return f"<ncclUniqueId object, ptr: {int(self)}>"

    def as_c_void_p(self):
        """Returns the data's address as `ctypes.c_void_p`
        Note:
            Implemented as function to not collide with
            autogenerated property names.
        """
        return ctypes.c_void_p(int(self))

    def __init__(self,*args,**kwargs):
        """Constructor type ncclUniqueId.

        Constructor for type ncclUniqueId.

        Args:
            *args:
                Positional arguments. Initialize all or a subset of the member variables
                according to their order of declaration.
            **kwargs:
                Can be used to initialize member variables at construction,
                Just pass an argument expression of the form <member>=<value>
                per member that you want to initialize.
        """
        ncclUniqueId.__allocate(&self._ptr)
        self._is_ptr_owner = True
        attribs = self.PROPERTIES()
        used_attribs = set()
        if len(args) > len(attribs):
            raise ValueError("More positional arguments specified than this type has properties.")
        for i,v in enumerate(args):
            setattr(self,attribs[i],v)
            used_attribs.add(attribs[i])
        valid_names = ", ".join(["'"+p+"'" for p in attribs])
        for k,v in kwargs.items():
            if k in used_attribs:
                raise KeyError(f"argument '{k}' has already been specified as positional argument.")
            elif k not in attribs:
                raise KeyError(f"'{k}' is no valid property name. Valid names: {valid_names}")
            setattr(self,k,v)
    def get_internal(self, i):
        """Get value of ``internal`` of ``(<crccl.ncclUniqueId*>self._ptr)[i]``.
        """
        return (<crccl.ncclUniqueId*>self._ptr)[i].internal
    # TODO add setters
    #def set_internal(self, i, char[128] value):
    #    """Set value ``internal`` of ``(<crccl.ncclUniqueId*>self._ptr)[i]``.
    #    """
    #    (<crccl.ncclUniqueId*>self._ptr)[i].internal = value
    @property
    def internal(self):
        """Opaque array>"""
        return self.get_internal(0)
    # TODO add setters
    #@internal.setter
    #def internal(self, char[128] value):
    #    self.set_internal(0,value)

    @staticmethod
    def PROPERTIES():
        return ["internal"]

    def __contains__(self,item):
        properties = self.PROPERTIES()
        return item in properties

    def __getitem__(self,item):
        properties = self.PROPERTIES()
        if isinstance(item,int):
            if item < 0 or item >= len(properties):
                raise IndexError()
            return getattr(self,properties[item])
        raise ValueError("'item' type must be 'int'")


class _ncclResult_t__Base(enum.IntEnum):
    """Empty enum base class that allows subclassing.
    """
    pass
class ncclResult_t(_ncclResult_t__Base):
    """Result type

    Attributes:
        ncclSuccess:
            No error
        ncclUnhandledCudaError:
            Unhandled HIP error
        ncclSystemError:
            Unhandled system error
        ncclInternalError:
            Internal Error - Please report to RCCL developers
        ncclInvalidArgument:
            Invalid argument
        ncclInvalidUsage:
            Invalid usage
        ncclRemoteError:
            Remote process exited or there was a network error
        ncclInProgress:
            RCCL operation in progress
        ncclNumResults:
            Number of result types
    """
    ncclSuccess = crccl.ncclSuccess
    ncclUnhandledCudaError = crccl.ncclUnhandledCudaError
    ncclSystemError = crccl.ncclSystemError
    ncclInternalError = crccl.ncclInternalError
    ncclInvalidArgument = crccl.ncclInvalidArgument
    ncclInvalidUsage = crccl.ncclInvalidUsage
    ncclRemoteError = crccl.ncclRemoteError
    ncclInProgress = crccl.ncclInProgress
    ncclNumResults = crccl.ncclNumResults
    @staticmethod
    def ctypes_type():
        """The type of the enum constants as ctypes type."""
        return ctypes.c_uint


cdef class ncclConfig_v21700(hip._util.types.Pointer):
    """Python wrapper for cdef class crccl.ncclConfig_v21700.

    Python wrapper for cdef class crccl.ncclConfig_v21700.

    If this type is initialized via its `__init__` method, it allocates a
    member of the underlying C type and destroys it again if the wrapper
    type is deallocated.

    This type also serves as adapter when appearing as argument type in a
    function signature. In this case, the type can further be initialized
    from a number of Python objects:

    * `None`:

      This will set the ``self._ptr`` attribute to ``NULL``.

    * `int`:

      Interprets the integer value as pointer address and writes it to ``self._ptr``.
      No ownership is transferred.

    * `ctypes.c_void_p`:

      Takes the pointer address ``pyobj.value`` and writes it to ``self._ptr``.
      No ownership is transferred.

    * `object` that implements the `CUDA Array Interface <https://numba.readthedocs.io/en/stable/cuda/cuda_array_interface.html>`_ protocol:

      Takes the integer-valued pointer address, i.e. the first entry of the `data` tuple
      from `pyobj`'s member ``__cuda_array_interface__``  and writes it to ``self._ptr``.

    * `object` that implements the Python buffer protocol:

      If the object represents a simple contiguous array,
      writes the `Py_buffer` associated with ``pyobj`` to `self._py_buffer`,
      sets the `self._py_buffer_acquired` flag to `True`, and
      writes `self._py_buffer.buf` to the data pointer `self._ptr`.

    * `hip._util.types.Pointer`:

      Takes the pointer address ``pyobj._ptr`` and writes it to ``self._ptr``.
      No ownership is transferred.

    Type checks are performed in the above order.

    C Attributes:
        _ptr (C type ``void *``, protected):
            Stores a pointer to the data of the original Python object.
        _is_ptr_owner (C type ``bint``, protected):
            If this wrapper is the owner of the underlying data.
        _py_buffer (C type ``Py_buffer`, protected):
            Stores a pointer to the data of the original Python object.
        _py_buffer_acquired (C type ``bint``, protected):
            Stores a pointer to the data of the original Python object.
    """
    # C members declared in pxd file

    def __cinit__(self):
        self._ptr = NULL
        self._is_ptr_owner = False
        self._py_buffer_acquired = False

    cdef crccl.ncclConfig_v21700* getElementPtr(self):
        return <crccl.ncclConfig_v21700*>self._ptr

    @staticmethod
    cdef ncclConfig_v21700 fromPtr(void* ptr, bint owner=False):
        """Factory function to create ``ncclConfig_v21700`` objects from
        given ``crccl.ncclConfig_v21700`` pointer.

        Setting ``owner`` flag to ``True`` causes
        the extension type to free the structure pointed to by ``ptr``
        when the wrapper object is deallocated.
        """
        # Fast call to __new__() that bypasses the __init__() constructor.
        cdef ncclConfig_v21700 wrapper = ncclConfig_v21700.__new__(ncclConfig_v21700)
        wrapper._ptr = ptr
        wrapper._is_ptr_owner = owner
        return wrapper

    @staticmethod
    def fromObj(pyobj):
        """Creates a ncclConfig_v21700 from a Python object.

        Derives a ncclConfig_v21700 from the given Python object ``pyobj``.
        In case ``pyobj`` is itself an ``ncclConfig_v21700`` reference, this method
        returns it directly. No new ``ncclConfig_v21700`` is created in this case.
        """
        return ncclConfig_v21700.fromPyobj(pyobj)

    @staticmethod
    cdef ncclConfig_v21700 fromPyobj(object pyobj):
        """Creates a ncclConfig_v21700 from a Python object.

        Derives a ncclConfig_v21700 from the given Python object ``pyobj``.
        In case ``pyobj`` is itself an ``ncclConfig_v21700`` reference, this method
        returns it directly. No new ``ncclConfig_v21700`` is created in this case.

        Args:
            pyobj (object): Must be either `None`; a `hip._util.types.Pointer`; a simple, contiguous buffer according to the buffer protocol;
                            or of type `ncclConfig_v21700`; `int`; or `ctypes.c_void_p`.

        Note:
            This routine does not perform a copy but returns the original ``pyobj``
            if ``pyobj`` is an instance of ncclConfig_v21700!
        """
        cdef ncclConfig_v21700 wrapper

        if isinstance(pyobj,ncclConfig_v21700):
            return pyobj
        else:
            wrapper = ncclConfig_v21700.__new__(ncclConfig_v21700)
            wrapper.init_from_pyobj(pyobj)
            return wrapper

    def __dealloc__(self):
        # Release the buffer handle
        if self._py_buffer_acquired is True:
            cpython.buffer.PyBuffer_Release(&self._py_buffer)
        # De-allocate if not null and flag is set
        if self._ptr is not NULL and self._is_ptr_owner is True:
            stdlib.free(self._ptr)
            self._ptr = NULL

    @staticmethod
    cdef __allocate(void** ptr):
        ptr[0] = stdlib.malloc(sizeof(crccl.ncclConfig_v21700))
        string.memset(<void*>ptr[0], 0, sizeof(crccl.ncclConfig_v21700))

        if ptr[0] is NULL:
            raise MemoryError

    @staticmethod
    cdef ncclConfig_v21700 new():
        """Factory function to create ncclConfig_v21700 objects with
        newly allocated crccl.ncclConfig_v21700"""
        cdef void* ptr
        ncclConfig_v21700.__allocate(&ptr)
        return ncclConfig_v21700.fromPtr(ptr, owner=True)

    @staticmethod
    cdef ncclConfig_v21700 fromValue(crccl.ncclConfig_v21700 other):
        """Allocate new C type and copy from ``other``.
        """
        wrapper = ncclConfig_v21700.new()
        string.memcpy(wrapper._ptr, &other, sizeof(crccl.ncclConfig_v21700))
        return wrapper

    def c_sizeof(self):
        """Returns the size of the underlying C type in bytes.
        Note:
            Implemented as function to not collide with
            autogenerated property names.
        """
        return sizeof(crccl.ncclConfig_v21700)

    def __int__(self):
        """Returns the data's address as long integer.
        """
        return cpython.long.PyLong_FromVoidPtr(self._ptr)

    def __repr__(self):
        return f"<ncclConfig_v21700 object, ptr: {int(self)}>"

    def as_c_void_p(self):
        """Returns the data's address as `ctypes.c_void_p`
        Note:
            Implemented as function to not collide with
            autogenerated property names.
        """
        return ctypes.c_void_p(int(self))

    def __init__(self,*args,**kwargs):
        """Constructor type ncclConfig_v21700.

        Constructor for type ncclConfig_v21700.

        Args:
            *args:
                Positional arguments. Initialize all or a subset of the member variables
                according to their order of declaration.
            **kwargs:
                Can be used to initialize member variables at construction,
                Just pass an argument expression of the form <member>=<value>
                per member that you want to initialize.
        """
        ncclConfig_v21700.__allocate(&self._ptr)
        self._is_ptr_owner = True
        attribs = self.PROPERTIES()
        used_attribs = set()
        if len(args) > len(attribs):
            raise ValueError("More positional arguments specified than this type has properties.")
        for i,v in enumerate(args):
            setattr(self,attribs[i],v)
            used_attribs.add(attribs[i])
        valid_names = ", ".join(["'"+p+"'" for p in attribs])
        for k,v in kwargs.items():
            if k in used_attribs:
                raise KeyError(f"argument '{k}' has already been specified as positional argument.")
            elif k not in attribs:
                raise KeyError(f"'{k}' is no valid property name. Valid names: {valid_names}")
            setattr(self,k,v)
    def get_size(self, i):
        """Get value ``size`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].size
    def set_size(self, i, unsigned long value):
        """Set value ``size`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].size = value
    @property
    def size(self):
        """Should not be touched"""
        return self.get_size(0)
    @size.setter
    def size(self, unsigned long value):
        self.set_size(0,value)

    def get_magic(self, i):
        """Get value ``magic`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].magic
    def set_magic(self, i, unsigned int value):
        """Set value ``magic`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].magic = value
    @property
    def magic(self):
        """Should not be touched"""
        return self.get_magic(0)
    @magic.setter
    def magic(self, unsigned int value):
        self.set_magic(0,value)

    def get_version(self, i):
        """Get value ``version`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].version
    def set_version(self, i, unsigned int value):
        """Set value ``version`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].version = value
    @property
    def version(self):
        """Should not be touched"""
        return self.get_version(0)
    @version.setter
    def version(self, unsigned int value):
        self.set_version(0,value)

    def get_blocking(self, i):
        """Get value ``blocking`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].blocking
    def set_blocking(self, i, int value):
        """Set value ``blocking`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].blocking = value
    @property
    def blocking(self):
        """Whether or not calls should block or not"""
        return self.get_blocking(0)
    @blocking.setter
    def blocking(self, int value):
        self.set_blocking(0,value)

    def get_cgaClusterSize(self, i):
        """Get value ``cgaClusterSize`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].cgaClusterSize
    def set_cgaClusterSize(self, i, int value):
        """Set value ``cgaClusterSize`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].cgaClusterSize = value
    @property
    def cgaClusterSize(self):
        """Cooperative group array cluster size"""
        return self.get_cgaClusterSize(0)
    @cgaClusterSize.setter
    def cgaClusterSize(self, int value):
        self.set_cgaClusterSize(0,value)

    def get_minCTAs(self, i):
        """Get value ``minCTAs`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].minCTAs
    def set_minCTAs(self, i, int value):
        """Set value ``minCTAs`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].minCTAs = value
    @property
    def minCTAs(self):
        """Minimum number of cooperative thread arrays (blocks)"""
        return self.get_minCTAs(0)
    @minCTAs.setter
    def minCTAs(self, int value):
        self.set_minCTAs(0,value)

    def get_maxCTAs(self, i):
        """Get value ``maxCTAs`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].maxCTAs
    def set_maxCTAs(self, i, int value):
        """Set value ``maxCTAs`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].maxCTAs = value
    @property
    def maxCTAs(self):
        """Maximum number of cooperative thread arrays (blocks)"""
        return self.get_maxCTAs(0)
    @maxCTAs.setter
    def maxCTAs(self, int value):
        self.set_maxCTAs(0,value)

    def get_netName(self, i):
        """Get value ``netName`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].netName
    def set_netName(self, i, const char * value):
        """Set value ``netName`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].netName = value
    @property
    def netName(self):
        """Force NCCL to use a specfic network"""
        return self.get_netName(0)
    @netName.setter
    def netName(self, const char * value):
        self.set_netName(0,value)

    def get_splitShare(self, i):
        """Get value ``splitShare`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        return (<crccl.ncclConfig_v21700*>self._ptr)[i].splitShare
    def set_splitShare(self, i, int value):
        """Set value ``splitShare`` of ``(<crccl.ncclConfig_v21700*>self._ptr)[i]``.
        """
        (<crccl.ncclConfig_v21700*>self._ptr)[i].splitShare = value
    @property
    def splitShare(self):
        """Allow communicators to share resources"""
        return self.get_splitShare(0)
    @splitShare.setter
    def splitShare(self, int value):
        self.set_splitShare(0,value)

    @staticmethod
    def PROPERTIES():
        return ["size","magic","version","blocking","cgaClusterSize","minCTAs","maxCTAs","netName","splitShare"]

    def __contains__(self,item):
        properties = self.PROPERTIES()
        return item in properties

    def __getitem__(self,item):
        properties = self.PROPERTIES()
        if isinstance(item,int):
            if item < 0 or item >= len(properties):
                raise IndexError()
            return getattr(self,properties[item])
        raise ValueError("'item' type must be 'int'")


ncclConfig_t = ncclConfig_v21700

@cython.embedsignature(True)
def ncclMemAlloc(object ptr, unsigned long size):
    r"""(No short description, might be part of a group.)

    @} */

    Args:
        ptr (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        size (`~.int`):
            (undocumented)
    """
    _ncclMemAlloc__retval = ncclResult_t(crccl.ncclMemAlloc(
        <void **>hip._util.types.Pointer.fromPyobj(ptr)._ptr,size))
    return (_ncclMemAlloc__retval,)


@cython.embedsignature(True)
def pncclMemAlloc(object ptr, unsigned long size):
    r"""(No short description, might be part of a group.)

    Args:
        ptr (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        size (`~.int`):
            (undocumented)
    """
    _pncclMemAlloc__retval = ncclResult_t(crccl.pncclMemAlloc(
        <void **>hip._util.types.Pointer.fromPyobj(ptr)._ptr,size))
    return (_pncclMemAlloc__retval,)


@cython.embedsignature(True)
def ncclMemFree(object ptr):
    r"""(No short description, might be part of a group.)

    Args:
        ptr (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)
    """
    _ncclMemFree__retval = ncclResult_t(crccl.ncclMemFree(
        <void *>hip._util.types.Pointer.fromPyobj(ptr)._ptr))
    return (_ncclMemFree__retval,)


@cython.embedsignature(True)
def pncclMemFree(object ptr):
    r"""(No short description, might be part of a group.)

    Args:
        ptr (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)
    """
    _pncclMemFree__retval = ncclResult_t(crccl.pncclMemFree(
        <void *>hip._util.types.Pointer.fromPyobj(ptr)._ptr))
    return (_pncclMemFree__retval,)


@cython.embedsignature(True)
def ncclGetVersion():
    r"""Return the RCCL_VERSION_CODE of RCCL in the supplied integer.

    This integer is coded with the MAJOR, MINOR and PATCH level of RCCL.

    Returns:
        A `~.tuple` of size 2 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
        * `~.int`:
                Pointer to where version will be stored
    """
    cdef int version
    _ncclGetVersion__retval = ncclResult_t(crccl.ncclGetVersion(&version))
    return (_ncclGetVersion__retval,version)


@cython.embedsignature(True)
def pncclGetVersion():
    r"""(No short description, might be part of a group.)

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * version (`~.int`):
            (undocumented)
    """
    cdef int version
    _pncclGetVersion__retval = ncclResult_t(crccl.pncclGetVersion(&version))
    return (_pncclGetVersion__retval,version)


@cython.embedsignature(True)
def ncclGetUniqueId(object uniqueId):
    r"""Generates an ID for ncclCommInitRank.

    Generates an ID to be used in ncclCommInitRank.
    ncclGetUniqueId should be called once by a single rank and the
    ID should be distributed to all ranks in the communicator before
    using it as a parameter for ncclCommInitRank.

    Args:
        uniqueId (`~.ncclUniqueId`/`~.object`) -- *OUT*:
            Pointer to where uniqueId will be stored

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclGetUniqueId__retval = ncclResult_t(crccl.ncclGetUniqueId(
        ncclUniqueId.fromPyobj(uniqueId).getElementPtr()))
    return (_ncclGetUniqueId__retval,)


@cython.embedsignature(True)
def pncclGetUniqueId(object uniqueId):
    r"""(No short description, might be part of a group.)

    Args:
        uniqueId (`~.ncclUniqueId`/`~.object`):
            (undocumented)
    """
    _pncclGetUniqueId__retval = ncclResult_t(crccl.pncclGetUniqueId(
        ncclUniqueId.fromPyobj(uniqueId).getElementPtr()))
    return (_pncclGetUniqueId__retval,)


@cython.embedsignature(True)
def ncclCommInitRankConfig(int nranks, object commId, int rank, object config):
    r"""Create a new communicator with config.

    Create a new communicator (multi thread/process version) with a configuration
    set by users. See ``rccl_config_type`` for more details.
    Each rank is associated to a CUDA device, which has to be set before calling
    ncclCommInitRank.

    Args:
        nranks (`~.int`) -- *IN*:
            Total number of ranks participating in this communicator

        commId (`~.ncclUniqueId`) -- *IN*:
            UniqueId required for initialization

        rank (`~.int`) -- *IN*:
            Current rank to create communicator for. [0 to nranks-1]

        config (`~.ncclConfig_v21700`/`~.object`) -- *IN*:
            Pointer to communicator configuration

    Returns:
        A `~.tuple` of size 2 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
        * `~.ncclComm`:
                Pointer to created communicator
    """
    comm = ncclComm.fromPtr(NULL)
    _ncclCommInitRankConfig__retval = ncclResult_t(crccl.ncclCommInitRankConfig(<crccl.ncclComm**>&comm._ptr,nranks,
        ncclUniqueId.fromPyobj(commId).getElementPtr()[0],rank,
        ncclConfig_v21700.fromPyobj(config).getElementPtr()))
    return (_ncclCommInitRankConfig__retval,None if comm._ptr == NULL else comm)


@cython.embedsignature(True)
def pncclCommInitRankConfig(int nranks, object commId, int rank, object config):
    r"""(No short description, might be part of a group.)

    Args:
        nranks (`~.int`):
            (undocumented)

        commId (`~.ncclUniqueId`):
            (undocumented)

        rank (`~.int`):
            (undocumented)

        config (`~.ncclConfig_v21700`/`~.object`):
            (undocumented)

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * comm (`~.ncclComm`):
            (undocumented)
    """
    comm = ncclComm.fromPtr(NULL)
    _pncclCommInitRankConfig__retval = ncclResult_t(crccl.pncclCommInitRankConfig(<crccl.ncclComm**>&comm._ptr,nranks,
        ncclUniqueId.fromPyobj(commId).getElementPtr()[0],rank,
        ncclConfig_v21700.fromPyobj(config).getElementPtr()))
    return (_pncclCommInitRankConfig__retval,None if comm._ptr == NULL else comm)


@cython.embedsignature(True)
def ncclCommInitRank(int nranks, object commId, int rank):
    r"""Creates a new communicator (multi thread/process version).

    Rank must be between 0 and nranks-1 and unique within a communicator clique.
    Each rank is associated to a CUDA device, which has to be set before calling
    ncclCommInitRank.  ncclCommInitRank implicitly syncronizes with other ranks,
    so it must be called by different threads/processes or use ncclGroupStart/ncclGroupEnd.

    Args:
        nranks (`~.int`) -- *IN*:
            Total number of ranks participating in this communicator

        commId (`~.ncclUniqueId`) -- *IN*:
            UniqueId required for initialization

        rank (`~.int`) -- *IN*:
            Current rank to create communicator for

    Returns:
        A `~.tuple` of size 2 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
        * `~.ncclComm`:
                Pointer to created communicator
    """
    comm = ncclComm.fromPtr(NULL)
    _ncclCommInitRank__retval = ncclResult_t(crccl.ncclCommInitRank(<crccl.ncclComm**>&comm._ptr,nranks,
        ncclUniqueId.fromPyobj(commId).getElementPtr()[0],rank))
    return (_ncclCommInitRank__retval,None if comm._ptr == NULL else comm)


@cython.embedsignature(True)
def pncclCommInitRank(int nranks, object commId, int rank):
    r"""(No short description, might be part of a group.)

    Args:
        nranks (`~.int`):
            (undocumented)

        commId (`~.ncclUniqueId`):
            (undocumented)

        rank (`~.int`):
            (undocumented)

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * comm (`~.ncclComm`):
            (undocumented)
    """
    comm = ncclComm.fromPtr(NULL)
    _pncclCommInitRank__retval = ncclResult_t(crccl.pncclCommInitRank(<crccl.ncclComm**>&comm._ptr,nranks,
        ncclUniqueId.fromPyobj(commId).getElementPtr()[0],rank))
    return (_pncclCommInitRank__retval,None if comm._ptr == NULL else comm)


@cython.embedsignature(True)
def ncclCommInitAll(object comm, int ndev, object devlist):
    r"""Creates a clique of communicators (single process version).

    This is a convenience function to create a single-process communicator clique.
    Returns an array of ndev newly initialized communicators in comm.
    comm should be pre-allocated with size at least ndev*sizeof(ncclComm_t).
    If devlist is NULL, the first ndev HIP devices are used.
    Order of devlist defines user-order of processors within the communicator.

    Args:
        comm (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Pointer to array of created communicators

        ndev (`~.int`) -- *IN*:
            Total number of ranks participating in this communicator

        devlist (`~.hip._util.types.ListOfInt`/`~.object`) -- *IN*:
            Array of GPU device indices to create for

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclCommInitAll__retval = ncclResult_t(crccl.ncclCommInitAll(
        <crccl.ncclComm_t*>hip._util.types.Pointer.fromPyobj(comm)._ptr,ndev,
        <const int *>hip._util.types.ListOfInt.fromPyobj(devlist)._ptr))
    return (_ncclCommInitAll__retval,)


@cython.embedsignature(True)
def pncclCommInitAll(object comm, int ndev, object devlist):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        ndev (`~.int`):
            (undocumented)

        devlist (`~.hip._util.types.ListOfInt`/`~.object`):
            (undocumented)
    """
    _pncclCommInitAll__retval = ncclResult_t(crccl.pncclCommInitAll(
        <crccl.ncclComm_t*>hip._util.types.Pointer.fromPyobj(comm)._ptr,ndev,
        <const int *>hip._util.types.ListOfInt.fromPyobj(devlist)._ptr))
    return (_pncclCommInitAll__retval,)


@cython.embedsignature(True)
def ncclCommFinalize(object comm):
    r"""Finalize a communicator.

    ncclCommFinalize flushes all issued communications
    and marks communicator state as ncclInProgress. The state will change to ncclSuccess
    when the communicator is globally quiescent and related resources are freed; then,
    calling ncclCommDestroy can locally free the rest of the resources (e.g. communicator
    itself) without blocking.

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to finalize

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclCommFinalize__retval = ncclResult_t(crccl.ncclCommFinalize(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_ncclCommFinalize__retval,)


@cython.embedsignature(True)
def pncclCommFinalize(object comm):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)
    """
    _pncclCommFinalize__retval = ncclResult_t(crccl.pncclCommFinalize(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_pncclCommFinalize__retval,)


@cython.embedsignature(True)
def ncclCommDestroy(object comm):
    r"""Frees local resources associated with communicator object.

    Destroy all local resources associated with the passed in communicator object

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to destroy

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclCommDestroy__retval = ncclResult_t(crccl.ncclCommDestroy(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_ncclCommDestroy__retval,)


@cython.embedsignature(True)
def pncclCommDestroy(object comm):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)
    """
    _pncclCommDestroy__retval = ncclResult_t(crccl.pncclCommDestroy(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_pncclCommDestroy__retval,)


@cython.embedsignature(True)
def ncclCommAbort(object comm):
    r"""Abort any in-progress calls and destroy the communicator object.

    Frees resources associated with communicator object and aborts any operations
    that might still be running on the device.

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to abort and destroy

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclCommAbort__retval = ncclResult_t(crccl.ncclCommAbort(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_ncclCommAbort__retval,)


@cython.embedsignature(True)
def pncclCommAbort(object comm):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)
    """
    _pncclCommAbort__retval = ncclResult_t(crccl.pncclCommAbort(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_pncclCommAbort__retval,)


@cython.embedsignature(True)
def ncclCommSplit(object comm, int color, int key, object config):
    r"""Create one or more communicators from an existing one.

    Creates one or more communicators from an existing one.
    Ranks with the same color will end up in the same communicator.
    Within the new communicator, key will be used to order ranks.
    NCCL_SPLIT_NOCOLOR as color will indicate the rank will not be part of any group
    and will therefore return a NULL communicator.
    If config is NULL, the new communicator will inherit the original communicator's configuration

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Original communicator object for this rank

        color (`~.int`) -- *IN*:
            Color to assign this rank

        key (`~.int`) -- *IN*:
            Key used to order ranks within the same new communicator

        config (`~.ncclConfig_v21700`/`~.object`) -- *IN*:
            Config file for new communicator. May be NULL to inherit from comm

    Returns:
        A `~.tuple` of size 2 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
        * `~.ncclComm`:
                Pointer to new communicator
    """
    newcomm = ncclComm.fromPtr(NULL)
    _ncclCommSplit__retval = ncclResult_t(crccl.ncclCommSplit(
        ncclComm.fromPyobj(comm).getElementPtr(),color,key,<crccl.ncclComm**>&newcomm._ptr,
        ncclConfig_v21700.fromPyobj(config).getElementPtr()))
    return (_ncclCommSplit__retval,None if newcomm._ptr == NULL else newcomm)


@cython.embedsignature(True)
def pncclCommSplit(object comm, int color, int key, object config):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        color (`~.int`):
            (undocumented)

        key (`~.int`):
            (undocumented)

        config (`~.ncclConfig_v21700`/`~.object`):
            (undocumented)

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * newcomm (`~.ncclComm`):
            (undocumented)
    """
    newcomm = ncclComm.fromPtr(NULL)
    _pncclCommSplit__retval = ncclResult_t(crccl.pncclCommSplit(
        ncclComm.fromPyobj(comm).getElementPtr(),color,key,<crccl.ncclComm**>&newcomm._ptr,
        ncclConfig_v21700.fromPyobj(config).getElementPtr()))
    return (_pncclCommSplit__retval,None if newcomm._ptr == NULL else newcomm)


@cython.embedsignature(True)
def ncclGetErrorString(object result):
    r"""Returns a string for each result code.

    Returns a human-readable string describing the given result code.

    Args:
        result (`~.ncclResult_t`) -- *IN*:
            Result code to get description for

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.bytes`: String containing description of result code.
    """
    if not isinstance(result,_ncclResult_t__Base):
        raise TypeError("argument 'result' must be of type '_ncclResult_t__Base'")
    _ncclGetErrorString__retval = hip._util.types.CStr.fromPtr(<void*>crccl.ncclGetErrorString(result.value))
    return (None if _ncclGetErrorString__retval._ptr == NULL else _ncclGetErrorString__retval,)


@cython.embedsignature(True)
def pncclGetErrorString(object result):
    r"""(No short description, might be part of a group.)

    Args:
        result (`~.ncclResult_t`):
            (undocumented)
    """
    if not isinstance(result,_ncclResult_t__Base):
        raise TypeError("argument 'result' must be of type '_ncclResult_t__Base'")
    _pncclGetErrorString__retval = hip._util.types.CStr.fromPtr(<void*>crccl.pncclGetErrorString(result.value))
    return (None if _pncclGetErrorString__retval._ptr == NULL else _pncclGetErrorString__retval,)


@cython.embedsignature(True)
def ncclGetLastError(object comm):
    r"""(No short description, might be part of a group.)

    */

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)
    """
    _ncclGetLastError__retval = hip._util.types.CStr.fromPtr(<void*>crccl.ncclGetLastError(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (None if _ncclGetLastError__retval._ptr == NULL else _ncclGetLastError__retval,)


@cython.embedsignature(True)
def pncclGetLastError(object comm):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)
    """
    _pncclGetLastError__retval = hip._util.types.CStr.fromPtr(<void*>crccl.pncclGetLastError(
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (None if _pncclGetLastError__retval._ptr == NULL else _pncclGetLastError__retval,)


@cython.embedsignature(True)
def ncclCommGetAsyncError(object comm, object asyncError):
    r"""Checks whether the comm has encountered any asynchronous errors

    Query whether the provided communicator has encountered any asynchronous errors

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to query

        asyncError (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Pointer to where result code will be stored

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclCommGetAsyncError__retval = ncclResult_t(crccl.ncclCommGetAsyncError(
        ncclComm.fromPyobj(comm).getElementPtr(),
        <crccl.ncclResult_t *>hip._util.types.Pointer.fromPyobj(asyncError)._ptr))
    return (_ncclCommGetAsyncError__retval,)


@cython.embedsignature(True)
def pncclCommGetAsyncError(object comm, object asyncError):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        asyncError (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)
    """
    _pncclCommGetAsyncError__retval = ncclResult_t(crccl.pncclCommGetAsyncError(
        ncclComm.fromPyobj(comm).getElementPtr(),
        <crccl.ncclResult_t *>hip._util.types.Pointer.fromPyobj(asyncError)._ptr))
    return (_pncclCommGetAsyncError__retval,)


@cython.embedsignature(True)
def ncclCommCount(object comm):
    r"""Gets the number of ranks in the communicator clique.

    Returns the number of ranks in the communicator clique (as set during initialization)

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to query

    Returns:
        A `~.tuple` of size 2 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
        * `~.int`:
                Pointer to where number of ranks will be stored
    """
    cdef int count
    _ncclCommCount__retval = ncclResult_t(crccl.ncclCommCount(
        ncclComm.fromPyobj(comm).getElementPtr(),&count))
    return (_ncclCommCount__retval,count)


@cython.embedsignature(True)
def pncclCommCount(object comm):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * count (`~.int`):
            (undocumented)
    """
    cdef int count
    _pncclCommCount__retval = ncclResult_t(crccl.pncclCommCount(
        ncclComm.fromPyobj(comm).getElementPtr(),&count))
    return (_pncclCommCount__retval,count)


@cython.embedsignature(True)
def ncclCommCuDevice(object comm):
    r"""Get the ROCm device index associated with a communicator

    Returns the ROCm device number associated with the provided communicator.

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to query

    Returns:
        A `~.tuple` of size 2 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
        * `~.int`:
                Pointer to where the associated ROCm device index will be stored
    """
    cdef int device
    _ncclCommCuDevice__retval = ncclResult_t(crccl.ncclCommCuDevice(
        ncclComm.fromPyobj(comm).getElementPtr(),&device))
    return (_ncclCommCuDevice__retval,device)


@cython.embedsignature(True)
def pncclCommCuDevice(object comm):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * device (`~.int`):
            (undocumented)
    """
    cdef int device
    _pncclCommCuDevice__retval = ncclResult_t(crccl.pncclCommCuDevice(
        ncclComm.fromPyobj(comm).getElementPtr(),&device))
    return (_pncclCommCuDevice__retval,device)


@cython.embedsignature(True)
def ncclCommUserRank(object comm):
    r"""Get the rank associated with a communicator

    Returns the user-ordered "rank" associated with the provided communicator.

    Args:
        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to query

    Returns:
        A `~.tuple` of size 2 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
        * `~.int`:
                Pointer to where the associated rank will be stored
    """
    cdef int rank
    _ncclCommUserRank__retval = ncclResult_t(crccl.ncclCommUserRank(
        ncclComm.fromPyobj(comm).getElementPtr(),&rank))
    return (_ncclCommUserRank__retval,rank)


@cython.embedsignature(True)
def pncclCommUserRank(object comm):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * rank (`~.int`):
            (undocumented)
    """
    cdef int rank
    _pncclCommUserRank__retval = ncclResult_t(crccl.pncclCommUserRank(
        ncclComm.fromPyobj(comm).getElementPtr(),&rank))
    return (_pncclCommUserRank__retval,rank)


@cython.embedsignature(True)
def ncclCommRegister(object comm, object buff, unsigned long size, object handle):
    r"""(No short description, might be part of a group.)

    */
    @} */

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        buff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        size (`~.int`):
            (undocumented)

        handle (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)
    """
    _ncclCommRegister__retval = ncclResult_t(crccl.ncclCommRegister(
        ncclComm.fromPyobj(comm).getElementPtr(),
        <void *>hip._util.types.Pointer.fromPyobj(buff)._ptr,size,
        <void **>hip._util.types.Pointer.fromPyobj(handle)._ptr))
    return (_ncclCommRegister__retval,)


@cython.embedsignature(True)
def pncclCommRegister(object comm, object buff, unsigned long size, object handle):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        buff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        size (`~.int`):
            (undocumented)

        handle (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)
    """
    _pncclCommRegister__retval = ncclResult_t(crccl.pncclCommRegister(
        ncclComm.fromPyobj(comm).getElementPtr(),
        <void *>hip._util.types.Pointer.fromPyobj(buff)._ptr,size,
        <void **>hip._util.types.Pointer.fromPyobj(handle)._ptr))
    return (_pncclCommRegister__retval,)


@cython.embedsignature(True)
def ncclCommDeregister(object comm, object handle):
    r"""(No short description, might be part of a group.)

    */

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        handle (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)
    """
    _ncclCommDeregister__retval = ncclResult_t(crccl.ncclCommDeregister(
        ncclComm.fromPyobj(comm).getElementPtr(),
        <void *>hip._util.types.Pointer.fromPyobj(handle)._ptr))
    return (_ncclCommDeregister__retval,)


@cython.embedsignature(True)
def pncclCommDeregister(object comm, object handle):
    r"""(No short description, might be part of a group.)

    Args:
        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        handle (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)
    """
    _pncclCommDeregister__retval = ncclResult_t(crccl.pncclCommDeregister(
        ncclComm.fromPyobj(comm).getElementPtr(),
        <void *>hip._util.types.Pointer.fromPyobj(handle)._ptr))
    return (_pncclCommDeregister__retval,)


class _ncclRedOp_dummy_t__Base(enum.IntEnum):
    """Empty enum base class that allows subclassing.
    """
    pass
class ncclRedOp_dummy_t(_ncclRedOp_dummy_t__Base):
    """Dummy reduction enumeration

    Attributes:
        ncclNumOps_dummy:
            (undocumented)
    """
    ncclNumOps_dummy = crccl.ncclNumOps_dummy
    @staticmethod
    def ctypes_type():
        """The type of the enum constants as ctypes type."""
        return ctypes.c_uint


class _ncclRedOp_t__Base(enum.IntEnum):
    """Empty enum base class that allows subclassing.
    """
    pass
class ncclRedOp_t(_ncclRedOp_t__Base):
    """Reduction operation selector

    Attributes:
        ncclSum:
            Sum
        ncclProd:
            Product
        ncclMax:
            Max
        ncclMin:
            Min
        ncclAvg:
            Average
        ncclNumOps:
            Number of built-in reduction ops
        ncclMaxRedOp:
            Largest value for ncclRedOp_t
    """
    ncclSum = crccl.ncclSum
    ncclProd = crccl.ncclProd
    ncclMax = crccl.ncclMax
    ncclMin = crccl.ncclMin
    ncclAvg = crccl.ncclAvg
    ncclNumOps = crccl.ncclNumOps
    ncclMaxRedOp = crccl.ncclMaxRedOp
    @staticmethod
    def ctypes_type():
        """The type of the enum constants as ctypes type."""
        return ctypes.c_uint


class _ncclDataType_t__Base(enum.IntEnum):
    """Empty enum base class that allows subclassing.
    """
    pass
class ncclDataType_t(_ncclDataType_t__Base):
    """Data types

    Attributes:
        ncclInt8:
            (undocumented)
        ncclChar:
            (undocumented)
        ncclUint8:
            (undocumented)
        ncclInt32:
            (undocumented)
        ncclInt:
            (undocumented)
        ncclUint32:
            (undocumented)
        ncclInt64:
            (undocumented)
        ncclUint64:
            (undocumented)
        ncclFloat16:
            (undocumented)
        ncclHalf:
            (undocumented)
        ncclFloat32:
            (undocumented)
        ncclFloat:
            (undocumented)
        ncclFloat64:
            (undocumented)
        ncclDouble:
            (undocumented)
        ncclBfloat16:
            (undocumented)
        ncclFp8E4M3:
            (undocumented)
        ncclFp8E5M2:
            (undocumented)
        ncclNumTypes:
            (undocumented)
    """
    ncclInt8 = crccl.ncclInt8
    ncclChar = crccl.ncclChar
    ncclUint8 = crccl.ncclUint8
    ncclInt32 = crccl.ncclInt32
    ncclInt = crccl.ncclInt
    ncclUint32 = crccl.ncclUint32
    ncclInt64 = crccl.ncclInt64
    ncclUint64 = crccl.ncclUint64
    ncclFloat16 = crccl.ncclFloat16
    ncclHalf = crccl.ncclHalf
    ncclFloat32 = crccl.ncclFloat32
    ncclFloat = crccl.ncclFloat
    ncclFloat64 = crccl.ncclFloat64
    ncclDouble = crccl.ncclDouble
    ncclBfloat16 = crccl.ncclBfloat16
    ncclFp8E4M3 = crccl.ncclFp8E4M3
    ncclFp8E5M2 = crccl.ncclFp8E5M2
    ncclNumTypes = crccl.ncclNumTypes
    @staticmethod
    def ctypes_type():
        """The type of the enum constants as ctypes type."""
        return ctypes.c_uint


class _ncclScalarResidence_t__Base(enum.IntEnum):
    """Empty enum base class that allows subclassing.
    """
    pass
class ncclScalarResidence_t(_ncclScalarResidence_t__Base):
    """Location and dereferencing logic for scalar arguments.

    Attributes:
        ncclScalarDevice:
            Scalar is in device-visible memory
        ncclScalarHostImmediate:
            Scalar is in host-visible memory
    """
    ncclScalarDevice = crccl.ncclScalarDevice
    ncclScalarHostImmediate = crccl.ncclScalarHostImmediate
    @staticmethod
    def ctypes_type():
        """The type of the enum constants as ctypes type."""
        return ctypes.c_uint


@cython.embedsignature(True)
def ncclRedOpCreatePreMulSum(object op, object scalar, object datatype, object residence, object comm):
    r"""Create a custom pre-multiplier reduction operator

    Creates a new reduction operator which pre-multiplies input values by a given
    scalar locally before reducing them with peer values via summation. For use
    only with collectives launched against *comm* and *datatype*. The
     residence* argument indicates how/when the memory pointed to by *scalar*
    will be dereferenced. Upon return, the newly created operator's handle
    is stored in *op*.

    Args:
        op (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Pointer to where newly created custom reduction operator is to be stored

        scalar (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Pointer to scalar value.

        datatype (`~.ncclDataType_t`) -- *IN*:
            Scalar value datatype

        residence (`~.ncclScalarResidence_t`) -- *IN*:
            Memory type of the scalar value

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator to associate with this custom reduction operator

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(residence,_ncclScalarResidence_t__Base):
        raise TypeError("argument 'residence' must be of type '_ncclScalarResidence_t__Base'")
    _ncclRedOpCreatePreMulSum__retval = ncclResult_t(crccl.ncclRedOpCreatePreMulSum(
        <crccl.ncclRedOp_t *>hip._util.types.Pointer.fromPyobj(op)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(scalar)._ptr,datatype.value,residence.value,
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_ncclRedOpCreatePreMulSum__retval,)


@cython.embedsignature(True)
def pncclRedOpCreatePreMulSum(object op, object scalar, object datatype, object residence, object comm):
    r"""(No short description, might be part of a group.)

    Args:
        op (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        scalar (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        residence (`~.ncclScalarResidence_t`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(residence,_ncclScalarResidence_t__Base):
        raise TypeError("argument 'residence' must be of type '_ncclScalarResidence_t__Base'")
    _pncclRedOpCreatePreMulSum__retval = ncclResult_t(crccl.pncclRedOpCreatePreMulSum(
        <crccl.ncclRedOp_t *>hip._util.types.Pointer.fromPyobj(op)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(scalar)._ptr,datatype.value,residence.value,
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_pncclRedOpCreatePreMulSum__retval,)


@cython.embedsignature(True)
def ncclRedOpDestroy(object op, object comm):
    r"""Destroy custom reduction operator

    Destroys the reduction operator *op*. The operator must have been created by
    ncclRedOpCreatePreMul with the matching communicator *comm*. An operator may be
    destroyed as soon as the last RCCL function which is given that operator returns.

    Args:
        op (`~.ncclRedOp_t`) -- *IN*:
            Custom reduction operator is to be destroyed

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator associated with this reduction operator

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _ncclRedOpDestroy__retval = ncclResult_t(crccl.ncclRedOpDestroy(op.value,
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_ncclRedOpDestroy__retval,)


@cython.embedsignature(True)
def pncclRedOpDestroy(object op, object comm):
    r"""(No short description, might be part of a group.)

    Args:
        op (`~.ncclRedOp_t`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)
    """
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _pncclRedOpDestroy__retval = ncclResult_t(crccl.pncclRedOpDestroy(op.value,
        ncclComm.fromPyobj(comm).getElementPtr()))
    return (_pncclRedOpDestroy__retval,)


@cython.embedsignature(True)
def ncclReduce(object sendbuff, object recvbuff, unsigned long count, object datatype, object op, int root, object comm, object stream):
    r"""Reduce

    Reduces data arrays of length *count* in *sendbuff* into *recvbuff* using *op*
    operation.
     recvbuff* may be NULL on all calls except for root device.
     root* is the rank (not the HIP device) where data will reside after the
     operation is complete.
    In-place operation will happen if sendbuff == recvbuff.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Local device data buffer to be reduced

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data buffer where result is stored (only for *root* rank).  May be null for other ranks.

        count (`~.int`) -- *IN*:
            Number of elements in every send buffer

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        op (`~.ncclRedOp_t`) -- *IN*:
            Reduction operator type

        root (`~.int`) -- *IN*:
            Rank where result data array will be stored

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _ncclReduce__retval = ncclResult_t(crccl.ncclReduce(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,op.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclReduce__retval,)


@cython.embedsignature(True)
def pncclReduce(object sendbuff, object recvbuff, unsigned long count, object datatype, object op, int root, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        count (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        op (`~.ncclRedOp_t`):
            (undocumented)

        root (`~.int`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _pncclReduce__retval = ncclResult_t(crccl.pncclReduce(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,op.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclReduce__retval,)


@cython.embedsignature(True)
def ncclBcast(object buff, unsigned long count, object datatype, int root, object comm, object stream):
    r"""(Deprecated) Broadcast (in-place)

    Copies *count* values from *root* to all other devices.
    root is the rank (not the CUDA device) where data resides before the
    operation is started.
    This operation is implicitly in-place.

    Args:
        buff (`~.hip._util.types.Pointer`/`~.object`) -- *IN,OUT*:
            Input array on *root* to be copied to other ranks.  Output array for all ranks.

        count (`~.int`) -- *IN*:
            Number of elements in data buffer

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        root (`~.int`) -- *IN*:
            Rank owning buffer to be copied to others

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclBcast__retval = ncclResult_t(crccl.ncclBcast(
        <void *>hip._util.types.Pointer.fromPyobj(buff)._ptr,count,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclBcast__retval,)


@cython.embedsignature(True)
def pncclBcast(object buff, unsigned long count, object datatype, int root, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        buff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        count (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        root (`~.int`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclBcast__retval = ncclResult_t(crccl.pncclBcast(
        <void *>hip._util.types.Pointer.fromPyobj(buff)._ptr,count,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclBcast__retval,)


@cython.embedsignature(True)
def ncclBroadcast(object sendbuff, object recvbuff, unsigned long count, object datatype, int root, object comm, object stream):
    r"""Broadcast

    Copies *count* values from *sendbuff* on *root* to *recvbuff* on all devices.
     root* is the rank (not the HIP device) where data resides before the operation is started.
     sendbuff* may be NULL on ranks other than *root*.
    In-place operation will happen if *sendbuff* == *recvbuff*.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Data array to copy (if *root*).  May be NULL for other ranks

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Data array to store received array

        count (`~.int`) -- *IN*:
            Number of elements in data buffer

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        root (`~.int`) -- *IN*:
            Rank of broadcast root

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclBroadcast__retval = ncclResult_t(crccl.ncclBroadcast(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclBroadcast__retval,)


@cython.embedsignature(True)
def pncclBroadcast(object sendbuff, object recvbuff, unsigned long count, object datatype, int root, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        count (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        root (`~.int`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclBroadcast__retval = ncclResult_t(crccl.pncclBroadcast(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclBroadcast__retval,)


@cython.embedsignature(True)
def ncclAllReduce(object sendbuff, object recvbuff, unsigned long count, object datatype, object op, object comm, object stream):
    r"""All-Reduce

    Reduces data arrays of length *count* in *sendbuff* using *op* operation, and
    leaves identical copies of result on each *recvbuff*.
    In-place operation will happen if sendbuff == recvbuff.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Input data array to reduce

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to store reduced result array

        count (`~.int`) -- *IN*:
            Number of elements in data buffer

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        op (`~.ncclRedOp_t`) -- *IN*:
            Reduction operator

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _ncclAllReduce__retval = ncclResult_t(crccl.ncclAllReduce(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,op.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclAllReduce__retval,)


@cython.embedsignature(True)
def pncclAllReduce(object sendbuff, object recvbuff, unsigned long count, object datatype, object op, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        count (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        op (`~.ncclRedOp_t`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _pncclAllReduce__retval = ncclResult_t(crccl.pncclAllReduce(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,op.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclAllReduce__retval,)


@cython.embedsignature(True)
def ncclReduceScatter(object sendbuff, object recvbuff, unsigned long recvcount, object datatype, object op, object comm, object stream):
    r"""Reduce-Scatter

    Reduces data in *sendbuff* using *op* operation and leaves reduced result
    scattered over the devices so that *recvbuff* on rank i will contain the i-th
    block of the result.
    Assumes sendcount is equal to nranks*recvcount, which means that *sendbuff*
    should have a size of at least nranks*recvcount elements.
    In-place operations will happen if recvbuff == sendbuff + rank * recvcount.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Input data array to reduce

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to store reduced result subarray

        recvcount (`~.int`) -- *IN*:
            Number of elements each rank receives

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        op (`~.ncclRedOp_t`) -- *IN*:
            Reduction operator

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _ncclReduceScatter__retval = ncclResult_t(crccl.ncclReduceScatter(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,recvcount,datatype.value,op.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclReduceScatter__retval,)


@cython.embedsignature(True)
def pncclReduceScatter(object sendbuff, object recvbuff, unsigned long recvcount, object datatype, object op, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvcount (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        op (`~.ncclRedOp_t`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")                    
    if not isinstance(op,_ncclRedOp_t__Base):
        raise TypeError("argument 'op' must be of type '_ncclRedOp_t__Base'")
    _pncclReduceScatter__retval = ncclResult_t(crccl.pncclReduceScatter(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,recvcount,datatype.value,op.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclReduceScatter__retval,)


@cython.embedsignature(True)
def ncclAllGather(object sendbuff, object recvbuff, unsigned long sendcount, object datatype, object comm, object stream):
    r"""All-Gather

    Each device gathers *sendcount* values from other GPUs into *recvbuff*,
    receiving data from rank i at offset i*sendcount.
    Assumes recvcount is equal to nranks*sendcount, which means that recvbuff
    should have a size of at least nranks*sendcount elements.
    In-place operations will happen if sendbuff == recvbuff + rank * sendcount.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Input data array to send

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to store the gathered result

        sendcount (`~.int`) -- *IN*:
            Number of elements each rank sends

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclAllGather__retval = ncclResult_t(crccl.ncclAllGather(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,sendcount,datatype.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclAllGather__retval,)


@cython.embedsignature(True)
def pncclAllGather(object sendbuff, object recvbuff, unsigned long sendcount, object datatype, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        sendcount (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclAllGather__retval = ncclResult_t(crccl.pncclAllGather(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,sendcount,datatype.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclAllGather__retval,)


@cython.embedsignature(True)
def ncclSend(object sendbuff, unsigned long count, object datatype, int peer, object comm, object stream):
    r"""Send

    Send data from *sendbuff* to rank *peer*.
    Rank *peer* needs to call ncclRecv with the same *datatype* and the same *count*
    as this rank.
    This operation is blocking for the GPU. If multiple ncclSend and ncclRecv operations
    need to progress concurrently to complete, they must be fused within a ncclGroupStart /
    ncclGroupEnd section.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Data array to send

        count (`~.int`) -- *IN*:
            Number of elements to send

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        peer (`~.int`) -- *IN*:
            Peer rank to send to

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclSend__retval = ncclResult_t(crccl.ncclSend(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,count,datatype.value,peer,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclSend__retval,)


@cython.embedsignature(True)
def pncclSend(object sendbuff, unsigned long count, object datatype, int peer, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        count (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        peer (`~.int`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclSend__retval = ncclResult_t(crccl.pncclSend(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,count,datatype.value,peer,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclSend__retval,)


@cython.embedsignature(True)
def ncclRecv(object recvbuff, unsigned long count, object datatype, int peer, object comm, object stream):
    r"""Receive

    Receive data from rank *peer* into *recvbuff*.
    Rank *peer* needs to call ncclSend with the same datatype and the same count
    as this rank.
    This operation is blocking for the GPU. If multiple ncclSend and ncclRecv operations
    need to progress concurrently to complete, they must be fused within a ncclGroupStart/
    ncclGroupEnd section.

    Args:
        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to receive

        count (`~.int`) -- *IN*:
            Number of elements to receive

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        peer (`~.int`) -- *IN*:
            Peer rank to send to

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclRecv__retval = ncclResult_t(crccl.ncclRecv(
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,peer,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclRecv__retval,)


@cython.embedsignature(True)
def pncclRecv(object recvbuff, unsigned long count, object datatype, int peer, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        count (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        peer (`~.int`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclRecv__retval = ncclResult_t(crccl.pncclRecv(
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,peer,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclRecv__retval,)


@cython.embedsignature(True)
def ncclGather(object sendbuff, object recvbuff, unsigned long sendcount, object datatype, int root, object comm, object stream):
    r"""Gather

    Root device gathers *sendcount* values from other GPUs into *recvbuff*,
    receiving data from rank i at offset i*sendcount.
    Assumes recvcount is equal to nranks*sendcount, which means that *recvbuff*
    should have a size of at least nranks*sendcount elements.
    In-place operations will happen if sendbuff == recvbuff + rank * sendcount.
     recvbuff* may be NULL on ranks other than *root*.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Data array to send

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to receive into on *root*.

        sendcount (`~.int`) -- *IN*:
            Number of elements to send per rank

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        root (`~.int`) -- *IN*:
            Rank that receives data from all other ranks

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclGather__retval = ncclResult_t(crccl.ncclGather(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,sendcount,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclGather__retval,)


@cython.embedsignature(True)
def pncclGather(object sendbuff, object recvbuff, unsigned long sendcount, object datatype, int root, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        sendcount (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        root (`~.int`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclGather__retval = ncclResult_t(crccl.pncclGather(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,sendcount,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclGather__retval,)


@cython.embedsignature(True)
def ncclScatter(object sendbuff, object recvbuff, unsigned long recvcount, object datatype, int root, object comm, object stream):
    r"""Scatter

    Scattered over the devices so that recvbuff on rank i will contain the i-th
    block of the data on root.
    Assumes sendcount is equal to nranks*recvcount, which means that *sendbuff*
    should have a size of at least nranks*recvcount elements.
    In-place operations will happen if recvbuff == sendbuff + rank * recvcount.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Data array to send (on *root* rank).  May be NULL on other ranks.

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to receive partial subarray into

        recvcount (`~.int`) -- *IN*:
            Number of elements to receive per rank

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        root (`~.int`) -- *IN*:
            Rank that scatters data to all other ranks

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclScatter__retval = ncclResult_t(crccl.ncclScatter(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,recvcount,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclScatter__retval,)


@cython.embedsignature(True)
def pncclScatter(object sendbuff, object recvbuff, unsigned long recvcount, object datatype, int root, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvcount (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        root (`~.int`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclScatter__retval = ncclResult_t(crccl.pncclScatter(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,recvcount,datatype.value,root,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclScatter__retval,)


@cython.embedsignature(True)
def ncclAllToAll(object sendbuff, object recvbuff, unsigned long count, object datatype, object comm, object stream):
    r"""All-To-All

    Device (i) send (j)th block of data to device (j) and be placed as (i)th
    block. Each block for sending/receiving has *count* elements, which means
    that *recvbuff* and *sendbuff* should have a size of nranks*count elements.
    In-place operation is NOT supported. It is the user's responsibility
    to ensure that sendbuff and recvbuff are distinct.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Data array to send (contains blocks for each other rank)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to receive (contains blocks from each other rank)

        count (`~.int`) -- *IN*:
            Number of elements to send between each pair of ranks

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclAllToAll__retval = ncclResult_t(crccl.ncclAllToAll(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclAllToAll__retval,)


@cython.embedsignature(True)
def pncclAllToAll(object sendbuff, object recvbuff, unsigned long count, object datatype, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        count (`~.int`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclAllToAll__retval = ncclResult_t(crccl.pncclAllToAll(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,count,datatype.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclAllToAll__retval,)


@cython.embedsignature(True)
def ncclAllToAllv(object sendbuff, object sendcounts, object sdispls, object recvbuff, object recvcounts, object rdispls, object datatype, object comm, object stream):
    r"""All-To-Allv

    Device (i) sends sendcounts[j] of data from offset sdispls[j]
    to device (j). At the same time, device (i) receives recvcounts[j] of data
    from device (j) to be placed at rdispls[j].
    sendcounts, sdispls, recvcounts and rdispls are all measured in the units
    of datatype, not bytes.
    In-place operation will happen if sendbuff == recvbuff.

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`) -- *IN*:
            Data array to send (contains blocks for each other rank)

        sendcounts (`~.hip._util.types.ListOfUnsignedLong`/`~.object`) -- *IN*:
            Array containing number of elements to send to each participating rank

        sdispls (`~.hip._util.types.ListOfUnsignedLong`/`~.object`) -- *IN*:
            Array of offsets into *sendbuff* for each participating rank

        recvbuff (`~.hip._util.types.Pointer`/`~.object`) -- *OUT*:
            Data array to receive (contains blocks from each other rank)

        recvcounts (`~.hip._util.types.ListOfUnsignedLong`/`~.object`) -- *IN*:
            Array containing number of elements to receive from each participating rank

        rdispls (`~.hip._util.types.ListOfUnsignedLong`/`~.object`) -- *IN*:
            Array of offsets into *recvbuff* for each participating rank

        datatype (`~.ncclDataType_t`) -- *IN*:
            Data buffer element datatype

        comm (`~.ncclComm`/`~.object`) -- *IN*:
            Communicator group object to execute on

        stream (`~.ihipStream_t`/`~.object`) -- *IN*:
            HIP stream to execute collective on

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _ncclAllToAllv__retval = ncclResult_t(crccl.ncclAllToAllv(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(sendcounts)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(sdispls)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(recvcounts)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(rdispls)._ptr,datatype.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_ncclAllToAllv__retval,)


@cython.embedsignature(True)
def pncclAllToAllv(object sendbuff, object sendcounts, object sdispls, object recvbuff, object recvcounts, object rdispls, object datatype, object comm, object stream):
    r"""(No short description, might be part of a group.)

    Args:
        sendbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        sendcounts (`~.hip._util.types.ListOfUnsignedLong`/`~.object`):
            (undocumented)

        sdispls (`~.hip._util.types.ListOfUnsignedLong`/`~.object`):
            (undocumented)

        recvbuff (`~.hip._util.types.Pointer`/`~.object`):
            (undocumented)

        recvcounts (`~.hip._util.types.ListOfUnsignedLong`/`~.object`):
            (undocumented)

        rdispls (`~.hip._util.types.ListOfUnsignedLong`/`~.object`):
            (undocumented)

        datatype (`~.ncclDataType_t`):
            (undocumented)

        comm (`~.ncclComm`/`~.object`):
            (undocumented)

        stream (`~.ihipStream_t`/`~.object`):
            (undocumented)
    """
    if not isinstance(datatype,_ncclDataType_t__Base):
        raise TypeError("argument 'datatype' must be of type '_ncclDataType_t__Base'")
    _pncclAllToAllv__retval = ncclResult_t(crccl.pncclAllToAllv(
        <const void *>hip._util.types.Pointer.fromPyobj(sendbuff)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(sendcounts)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(sdispls)._ptr,
        <void *>hip._util.types.Pointer.fromPyobj(recvbuff)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(recvcounts)._ptr,
        <const unsigned long*>hip._util.types.ListOfUnsignedLong.fromPyobj(rdispls)._ptr,datatype.value,
        ncclComm.fromPyobj(comm).getElementPtr(),
        ihipStream_t.fromPyobj(stream).getElementPtr()))
    return (_pncclAllToAllv__retval,)


@cython.embedsignature(True)
def ncclGroupStart():
    r"""Group Start

    Start a group call. All calls to RCCL until ncclGroupEnd will be fused into
    a single RCCL operation. Nothing will be started on the HIP stream until
    ncclGroupEnd.

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclGroupStart__retval = ncclResult_t(crccl.ncclGroupStart())
    return (_ncclGroupStart__retval,)


@cython.embedsignature(True)
def pncclGroupStart():
    r"""(No short description, might be part of a group.)
    """
    _pncclGroupStart__retval = ncclResult_t(crccl.pncclGroupStart())
    return (_pncclGroupStart__retval,)


@cython.embedsignature(True)
def ncclGroupEnd():
    r"""Group End

    End a group call. Start a fused RCCL operation consisting of all calls since
    ncclGroupStart. Operations on the HIP stream depending on the RCCL operations
    need to be called after ncclGroupEnd.

    Returns:
        A `~.tuple` of size 1 that contains (in that order):

        * `~.ncclResult_t`: Result code. See ``rccl_result_code`` for more details.
    """
    _ncclGroupEnd__retval = ncclResult_t(crccl.ncclGroupEnd())
    return (_ncclGroupEnd__retval,)


@cython.embedsignature(True)
def pncclGroupEnd():
    r"""(No short description, might be part of a group.)
    """
    _pncclGroupEnd__retval = ncclResult_t(crccl.pncclGroupEnd())
    return (_pncclGroupEnd__retval,)

__all__ = [
    "NCCL_MAJOR",
    "NCCL_MINOR",
    "NCCL_PATCH",
    "NCCL_SUFFIX",
    "NCCL_VERSION_CODE",
    "RCCL_BFLOAT16",
    "RCCL_GATHER_SCATTER",
    "RCCL_ALLTOALLV",
    "NCCL_UNIQUE_ID_BYTES",
    "ncclComm",
    "ncclComm_t",
    "ncclUniqueId",
    "_ncclResult_t__Base",
    "ncclResult_t",
    "ncclConfig_v21700",
    "ncclConfig_t",
    "ncclMemAlloc",
    "pncclMemAlloc",
    "ncclMemFree",
    "pncclMemFree",
    "ncclGetVersion",
    "pncclGetVersion",
    "ncclGetUniqueId",
    "pncclGetUniqueId",
    "ncclCommInitRankConfig",
    "pncclCommInitRankConfig",
    "ncclCommInitRank",
    "pncclCommInitRank",
    "ncclCommInitAll",
    "pncclCommInitAll",
    "ncclCommFinalize",
    "pncclCommFinalize",
    "ncclCommDestroy",
    "pncclCommDestroy",
    "ncclCommAbort",
    "pncclCommAbort",
    "ncclCommSplit",
    "pncclCommSplit",
    "ncclGetErrorString",
    "pncclGetErrorString",
    "ncclGetLastError",
    "pncclGetLastError",
    "ncclCommGetAsyncError",
    "pncclCommGetAsyncError",
    "ncclCommCount",
    "pncclCommCount",
    "ncclCommCuDevice",
    "pncclCommCuDevice",
    "ncclCommUserRank",
    "pncclCommUserRank",
    "ncclCommRegister",
    "pncclCommRegister",
    "ncclCommDeregister",
    "pncclCommDeregister",
    "_ncclRedOp_dummy_t__Base",
    "ncclRedOp_dummy_t",
    "_ncclRedOp_t__Base",
    "ncclRedOp_t",
    "_ncclDataType_t__Base",
    "ncclDataType_t",
    "_ncclScalarResidence_t__Base",
    "ncclScalarResidence_t",
    "ncclRedOpCreatePreMulSum",
    "pncclRedOpCreatePreMulSum",
    "ncclRedOpDestroy",
    "pncclRedOpDestroy",
    "ncclReduce",
    "pncclReduce",
    "ncclBcast",
    "pncclBcast",
    "ncclBroadcast",
    "pncclBroadcast",
    "ncclAllReduce",
    "pncclAllReduce",
    "ncclReduceScatter",
    "pncclReduceScatter",
    "ncclAllGather",
    "pncclAllGather",
    "ncclSend",
    "pncclSend",
    "ncclRecv",
    "pncclRecv",
    "ncclGather",
    "pncclGather",
    "ncclScatter",
    "pncclScatter",
    "ncclAllToAll",
    "pncclAllToAll",
    "ncclAllToAllv",
    "pncclAllToAllv",
    "ncclGroupStart",
    "pncclGroupStart",
    "ncclGroupEnd",
    "pncclGroupEnd",
]